@@ -31,6 +31,8 @@ AccountDialog::AccountDialog(PumpController *pumpController, QWidget *parent) :
 
     this->pController = pumpController;
 
+    connect(pController, SIGNAL(openingAuthorizeURL(QUrl)),
+            this, SLOT(showAuthorizationURL(QUrl)));
     connect(pController, SIGNAL(authorizationStatusChanged(bool)),
             this, SLOT(showAuthorizationStatus(bool)));
 
@@ -71,11 +73,12 @@ AccountDialog::AccountDialog(PumpController *pumpController, QWidget *parent) :
             errorsLabel, SLOT(clear()));
 
 
-    authorizationStatusLabel = new QLabel("");
+    authorizationStatusLabel = new QLabel();
+    authorizationStatusLabel->setTextInteractionFlags(Qt::TextSelectableByKeyboard | Qt::TextSelectableByMouse);
 
 
     saveButton = new QPushButton(QIcon::fromTheme("document-save"),
-                                 tr("&Save details"));
+                                 tr("&Save Details"));
     connect(saveButton, SIGNAL(clicked()),
             this, SLOT(saveDetails()));
     cancelButton = new QPushButton(QIcon::fromTheme("dialog-cancel"),
@@ -194,6 +197,22 @@ void AccountDialog::showAuthorizationStatus(bool authorized)
 }
 
 
+/*
+ * Show the authorization URL in a label,
+ * in case the browser doesn't open automatically
+ *
+ */
+void AccountDialog::showAuthorizationURL(QUrl url)
+{
+    QString message = tr("If the browser doesn't open automatically, "
+                         "copy this address manually")
+                      + ":\n"
+                      + url.toString()
+                      + "\n";
+    this->authorizationStatusLabel->setText(message);
+}
+
+
 
 
 
