@@ -29,9 +29,24 @@ Composer::Composer(bool forPublisher)
     this->setTabChangesFocus(true);
 
 
-
     startConversationLabel = new QLabel(tr("Click here or press Control+N to post a note..."));
 
+    // A menu to insert some Unicode symbols
+    symbolsMenu = new QMenu(tr("Symbols"));
+    symbolsMenu->setIcon(QIcon::fromTheme("character-set"));
+    symbolsMenu->addAction(QString::fromUtf8("\342\230\272")); // Smiling face
+    symbolsMenu->addAction(QString::fromUtf8("\342\230\271")); // Sad face
+    symbolsMenu->addAction(QString::fromUtf8("\342\231\245")); // Heart
+    symbolsMenu->addAction(QString::fromUtf8("\342\231\253")); // Musical note
+    symbolsMenu->addAction(QString::fromUtf8("\342\230\225")); // Coffee
+    symbolsMenu->addAction(QString::fromUtf8("\342\234\224")); // Check mark
+    symbolsMenu->addAction(QString::fromUtf8("\342\230\205")); // Black star
+    symbolsMenu->addAction(QString::fromUtf8("\342\254\205")); // Arrow to the left
+    symbolsMenu->addAction(QString::fromUtf8("\342\236\241")); // Arrow to the right
+    connect(symbolsMenu, SIGNAL(triggered(QAction*)),
+            this, SLOT(insertSymbol(QAction*)));
+
+
     toolsMenu = new QMenu(tr("Formatting"));
     toolsMenu->addAction(QIcon::fromTheme(""),
                          tr("Normal"),
@@ -74,6 +89,10 @@ Composer::Composer(bool forPublisher)
 
     toolsMenu->addSeparator();
 
+    toolsMenu->addMenu(symbolsMenu);
+
+    toolsMenu->addSeparator();
+
     toolsMenu->addAction(QIcon::fromTheme("dialog-cancel"),
                          tr("Cancel Post"),
                          this,
@@ -272,7 +291,7 @@ void Composer::makePreformatted()
     if (!selectedText.isEmpty())
     {
         this->textCursor().removeSelectedText();
-        this->insertHtml("<pre>" + selectedText + "</pre>");
+        this->insertHtml("<pre>" + selectedText + "</pre> ");
     }
 
     this->setFocus();
@@ -359,6 +378,16 @@ void Composer::insertImage()
 
 
 
+void Composer::insertSymbol(QAction *action)
+{
+    this->insertPlainText(action->text());
+    this->insertPlainText(" ");
+
+    this->setFocus();
+}
+
+
+
 
 /*
  * Cancel editing of the post, clear it, return to minimum mode
