@@ -22,7 +22,7 @@
 
 ConfigDialog::ConfigDialog(int updateInterval, int postsPerPage, int tabsPosition,
                            bool tabsMovable, int publisherType, bool publicPosts,
-                           int showNotifications, QWidget *parent)
+                           int showNotifications, QWidget *parent) : QWidget(parent)
 {
     this->setWindowTitle("Dianara - " + tr("Program Configuration"));
     this->setWindowIcon(QIcon::fromTheme("configure"));
