@@ -23,7 +23,9 @@
 
 #include <QWidget>
 #include <QIcon>
-#include <QGridLayout>
+#include <QFormLayout>
+#include <QVBoxLayout>
+#include <QHBoxLayout>
 #include <QLabel>
 #include <QSpinBox>
 #include <QComboBox>
@@ -42,7 +44,9 @@ public:
                  int tabsPosition,
                  bool tabsMovable,
                  int publisherType,
-                 int showNotifications);
+                 bool publicPosts,
+                 int showNotifications,
+                 QWidget *parent);
     ~ConfigDialog();
 
 
@@ -52,6 +56,7 @@ signals:
                               int newTabsPosition,
                               bool newTabsMovable,
                               int newPublisherType,
+                              bool newPublicPosts,
                               int newShowNotifications);
 
 
@@ -60,27 +65,20 @@ public slots:
 
 
 private:
-    QGridLayout *mainLayout;
+    QVBoxLayout *mainLayout;
+
+    QFormLayout *optionsLayout;
 
-    QLabel *updateIntervalLabel;
     QSpinBox *updateIntervalSpinbox;
-
-    QLabel *postsPerPageLabel;
     QSpinBox *postsPerPageSpinbox;
-
-    QLabel *tabsPositionLabel;
     QComboBox *tabsPositionCombobox;
-
-    QLabel *tabsMovableLabel;
     QCheckBox *tabsMovableCheckbox;
-
-    QLabel *publisherTypeLabel;
     QComboBox *publisherTypeCombobox;
-
-    QLabel *showNotificationsLabel;
+    QCheckBox *publicPostsCheckbox;
     QComboBox *showNotificationsCombobox;
 
 
+    QHBoxLayout *buttonsLayout;
     QPushButton *saveConfigButton;
     QPushButton *cancelButton;
 };
