@@ -36,10 +36,10 @@ ContactCard::ContactCard(PumpController *pumpController,
     avatarLabel = new QLabel();
 
     // Get local file name, which is stored in base64 hash form
-    QString avatarFilename = MiscHelpers::getCachedAvatarFilename(contactData.value("avatar"));
+    QString avatarFilename = MiscHelpers::getCachedAvatarFilename(contactData["avatar"]);
 
     QFile avatarFile(avatarFilename);
-    if (avatarFile.exists(avatarFilename) && !contactData.value("avatar").isEmpty())
+    if (avatarFile.exists(avatarFilename) && !contactData["avatar"].isEmpty())
     {
         // Load avatar if already cached
         avatarLabel->setPixmap(QPixmap(avatarFilename).scaled(48, 48,
@@ -54,7 +54,7 @@ ContactCard::ContactCard(PumpController *pumpController,
         qDebug() << "ContactCard: Using placeholder";
 
         // Download avatar for next time (if empty, pumpController will do nothing)
-        pController->getAvatar(contactData.value("avatar"));
+        pController->getAvatar(contactData["avatar"]);
     }
 
 
@@ -64,29 +64,33 @@ ContactCard::ContactCard(PumpController *pumpController,
     nameFont.setBold(true);
     nameFont.setUnderline(true);
 
-    QString contactNameString = contactData.value("name");
+    QString contactNameString = contactData["name"];
 
     nameLabel = new QLabel();
     nameLabel->setText(contactNameString);
     nameLabel->setFont(nameFont);
 
 
-    this->contactID = contactData.value("id");
+    this->contactID = contactData["id"];
 
     tmpLabel = new QLabel();
+    tmpLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
     tmpLabel->setText(QString("<%1>\n").arg(contactID)
-                      + tr("Hometown") + QString(": %2").arg(contactData.value("hometown")));
+                      + tr("Hometown") + QString(": %2").arg(contactData["hometown"]));
     tmpLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+    tmpLabel->setWordWrap(true);
 
 
     // Bio as tooltip for the whole contact card
-    QString contactBio = contactData.value("bio");
+    QString contactBio = contactData["bio"];
     if (!contactBio.isEmpty())
     {
         // Make it rich text, so that it gets wordwrap
         this->setToolTip("<b><u>"
                          + tr("Bio for %1",
-                              "Abbreviation for Biography, but you can use the full word; %1=contact name")
+                              "Abbreviation for Biography, "
+                              "but you can use the full word; "
+                              "%1=contact name")
                            .arg(contactNameString)
                          + "</u></b>"
                          "<br><br>"
@@ -94,7 +98,8 @@ ContactCard::ContactCard(PumpController *pumpController,
     }
     else
     {
-        this->setToolTip(tr("No biography for %1", "%1=contact name").arg(contactNameString));
+        this->setToolTip(tr("No biography for %1",
+                            "%1=contact name").arg(contactNameString));
     }
 
 
@@ -103,7 +108,7 @@ ContactCard::ContactCard(PumpController *pumpController,
 
     followButton = new QPushButton("--");
     followButton->setFlat(true);    
-    if (contactData.value("following") == "false")
+    if (contactData["following"] == "false")
     {
         this->setButtonToFollow();
     }
