@@ -48,10 +48,13 @@ public:
                 QWidget *parent = 0);
     ~ContactCard();
 
+    void setButtonToFollow();
+    void setButtonToUnfollow();
 
 signals:
 
 public slots:
+    void followContact();
     void unfollowContact();
 
 private:
