@@ -36,6 +36,7 @@ ContactList::ContactList(PumpController *pumpController,
     addressLineEdit = new QLineEdit();
     addressLineEdit->setPlaceholderText(webfingerHelpMessage);
     addressLineEdit->setToolTip(webfingerHelpMessage);
+    this->addressLineEdit->setFocusPolicy(Qt::StrongFocus); // Prevents initial focus, somehow
 
     followButton = new QPushButton(QIcon::fromTheme("list-add-user"), tr("&Follow"));
     connect(followButton, SIGNAL(clicked()),
@@ -44,6 +45,7 @@ ContactList::ContactList(PumpController *pumpController,
     topLayout->addWidget(followButton);
 
     mainLayout->addLayout(topLayout);
+    mainLayout->addSpacing(8);
 
 
     // Widget for list of 'following'
@@ -93,11 +95,30 @@ ContactList::ContactList(PumpController *pumpController,
     listsScrollArea->setWidgetResizable(true);
 
 
-    reloadFollowersButton = new QPushButton(QIcon::fromTheme("view-refresh"),
-                                            tr("Refresh"));
-    reloadFollowersButton->setToolTip(tr("Reload list of followers"));
-    connect(reloadFollowersButton, SIGNAL(clicked()),
-            this, SLOT(refreshFollowers()));
+    // Options menu
+    optionsMenu = new QMenu("options-menu");
+    optionsMenu->addAction(QIcon::fromTheme("view-refresh"),
+                           tr("Reload Following"),
+                           this,
+                           SLOT(refreshFollowing()));
+    optionsMenu->addAction(QIcon::fromTheme("view-refresh"),
+                           tr("Reload Followers"),
+                           this,
+                           SLOT(refreshFollowers()));
+    optionsMenu->addSeparator();
+    optionsMenu->addAction(QIcon::fromTheme("document-export"),
+                           tr("Export Following"),
+                           this,
+                           SLOT(exportFollowing()));
+    optionsMenu->addAction(QIcon::fromTheme("document-export"),
+                           tr("Export Followers"),
+                           this,
+                           SLOT(exportFollowers()));
+
+
+    optionsButton = new QPushButton(QIcon::fromTheme("user-properties"),
+                                    tr("Optio&ns"));
+    optionsButton->setMenu(optionsMenu);
 
 
     this->tabWidget = new QTabWidget();
@@ -113,7 +134,7 @@ ContactList::ContactList(PumpController *pumpController,
                       QIcon::fromTheme("preferences-contact-list"),
                       "lists");
 
-    tabWidget->setCornerWidget(this->reloadFollowersButton);
+    tabWidget->setCornerWidget(this->optionsButton);
 
 
     this->followingCount = 0;
@@ -126,6 +147,8 @@ ContactList::ContactList(PumpController *pumpController,
     this->setLayout(mainLayout);
 
 
+    this->followButton->setFocus(); // Set focus here, away from addressLineEdit
+
     qDebug() << "Contact list created";
 }
 
@@ -145,7 +168,9 @@ void ContactList::clearFollowingListContents()
     {
         child->widget()->deleteLater(); // Delete the widget (contactCard) itself
         delete child;
-    }    
+    }
+
+    this->followingListString.clear();
 }
 
 void ContactList::clearFollowersListContents()
@@ -157,6 +182,8 @@ void ContactList::clearFollowersListContents()
         child->widget()->deleteLater();
         delete child;
     }
+
+    this->followersListString.clear();
 }
 
 
@@ -173,6 +200,49 @@ void ContactList::setTabLabels()
 }
 
 
+/*
+ * Write the list of contacts (following or followers)
+ * to a file selected by the user
+ *
+ */
+void ContactList::exportContactsToFile(QString listType)
+{
+    QString dialogTitle = listType == "following" ?
+                    tr("Export list of 'following' to a file") :
+                    tr("Export list of 'followers' to a file");
+
+    QString suggestedFilename = "dianara-"
+                                + pController->currentUsername()
+                                + "-"
+                                + listType;
+
+    QString filename = QFileDialog::getSaveFileName(this, dialogTitle,
+                                                    QDir::homePath() + "/" + suggestedFilename,
+                                                    "").trimmed();
+
+    if (filename.isEmpty()) // If dialog canceled, do nothing
+    {
+        return;
+    }
+
+
+    qDebug() << "Exporting to:"  << filename;
+
+    QFile exportFile(filename);
+    exportFile.open(QIODevice::WriteOnly);
+
+    if (listType == "following")
+    {
+        exportFile.write(this->followingListString.toLocal8Bit());
+    }
+    else // "followers"
+    {
+        exportFile.write(this->followersListString.toLocal8Bit());
+    }
+    exportFile.close();
+}
+
+
 
 
 
@@ -200,36 +270,45 @@ void ContactList::setContactListContents(QString listType, QVariantList contactL
 
 
     QMap<QString,QString> contactData;
+    QString contactInfoLineString;
 
     foreach (QVariant contact, contactList)
     {
-        contactData.insert("id", contact.toMap()["id"].toString().remove(0,5)); // remove "acct:"
-        contactData.insert("name", contact.toMap()["displayName"].toString());
-        contactData.insert("avatar", contact.toMap()["image"].toMap().value("url").toString());
-        contactData.insert("hometown", contact.toMap()["location"].toMap().value("displayName").toString());
-        contactData.insert("bio", contact.toMap()["summary"].toString());
+        QVariantMap contactMap = contact.toMap();
 
-        contactData.insert("following", contact.toMap()["pump_io"].toMap().value("followed").toString());
+        contactData.insert("id", contactMap["id"].toString().remove(0,5)); // remove "acct:"
+        contactData.insert("name", contactMap["displayName"].toString());
+        contactData.insert("avatar", contactMap["image"].toMap().value("url").toString());
+        contactData.insert("hometown", contactMap["location"].toMap().value("displayName").toString());
+        contactData.insert("bio", contactMap["summary"].toString());
+
+        contactData.insert("following", contactMap["pump_io"].toMap().value("followed").toString());
+
+        contactInfoLineString = contactData["name"]
+                              + "  <"
+                              + contactData["id"]
+                              + ">\n";
 
         if (listType == "following")
         {
             this->followingLayout->addWidget(new ContactCard(this->pController,
                                                             contactData,
                                                             this));
+            followingListString.append(contactInfoLineString);
+
         }
         else // == "followers"
         {
             this->followersLayout->addWidget(new ContactCard(this->pController,
                                                             contactData,
                                                             this));
+            followersListString.append(contactInfoLineString);
         }
     }
 
 
     // Update tab labels with number of following or followers, which were updated before
     this->setTabLabels();
-
-    this->followersWidget->setFocus(); // Set focus here, away from addressLineEdit
 }
 
 
@@ -275,17 +354,51 @@ void ContactList::setListsListContents(QVariantList listsList)
 }
 
 
+
+/*
+ * Ask for the updated list of Following
+ *
+ */
+void ContactList::refreshFollowing()
+{
+    qDebug() << "Refreshing list of Following...";
+    this->pController->getContactList("following");
+}
+
+
 /*
  * Ask for the updated list of followers
  *
  */
 void ContactList::refreshFollowers()
 {
-    qDebug() << "Refreshing list of followers...";
+    qDebug() << "Refreshing list of Followers...";
     this->pController->getContactList("followers");
 }
 
 
+/*
+ * Export list of "Following" to a text file
+ *
+ */
+void ContactList::exportFollowing()
+{
+    qDebug() << "Exporting Following...";
+    exportContactsToFile("following");
+}
+
+
+/*
+ * Export list of "Followers" to a text file
+ *
+ */
+void ContactList::exportFollowers()
+{
+    qDebug() << "Exporting Followers...";
+    exportContactsToFile("followers");
+}
+
+
 
 
 /*
@@ -311,16 +424,20 @@ void ContactList::followContact()
     {
         address.remove("https://");
 
-        QStringList addressParts = address.split("/");
-        address = addressParts.at(1) + "@" + addressParts.at(0);
+        if (address.contains("/")) // Very basic sanity check
+        {
+            QStringList addressParts = address.split("/");
+            address = addressParts.at(1) + "@" + addressParts.at(0);
 
-        validAddress = true;
-    }
-    else
-    {
-        qDebug() << "Invalid webfinger address!";
+            validAddress = true;
+        }
+        else
+        {
+            qDebug() << "Invalid webfinger address!";
+        }
     }
 
+
     if (validAddress)
     {
         qDebug() << "About to follow this address:" << address;
