@@ -32,6 +32,8 @@
 #include <QLineEdit>
 #include <QTabWidget>
 #include <QScrollArea>
+#include <QMenu>
+#include <QFileDialog>
 
 #include <QDebug>
 
@@ -52,6 +54,8 @@ public:
 
     void setTabLabels();
 
+    void exportContactsToFile(QString listType);
+
 signals:
 
 public slots:
@@ -59,8 +63,12 @@ public slots:
                                 QVariantList contactList);
     void setListsListContents(QVariantList listsList);
 
+    void refreshFollowing();
     void refreshFollowers();
 
+    void exportFollowing();
+    void exportFollowers();
+
     void followContact();
 
 private:
@@ -76,12 +84,14 @@ private:
     QVBoxLayout *followingLayout;
     QScrollArea *followingScrollArea;
     int followingCount;
+    QString followingListString;
 
 
     QWidget *followersWidget;
     QVBoxLayout *followersLayout;
     QScrollArea *followersScrollArea;
     int followersCount;
+    QString followersListString;
 
 
     QLabel *listsLabel;
@@ -90,7 +100,8 @@ private:
 
 
 
-    QPushButton *reloadFollowersButton;
+    QPushButton *optionsButton;
+    QMenu *optionsMenu;
 
     PumpController *pController;
 };
