@@ -26,6 +26,10 @@
 #include <QCloseEvent>
 #include <QKeyEvent>
 #include <QDesktopWidget>
+#include <QMenu>
+#include <QAction>
+#include <QFileDialog>
+#include <QMessageBox>
 
 #include <QDebug>
 
@@ -38,22 +42,27 @@ public:
                          QString title,
                          QWidget *parent = 0);
     ~ImageViewer();
+    void createContextMenu();
 
 
 signals:
 
 
 public slots:
+    void saveImage();
 
 
 protected:
     virtual void closeEvent(QCloseEvent *event);
     virtual void keyPressEvent(QKeyEvent *event);
+    virtual void contextMenuEvent(QContextMenuEvent *event);
 
 
 private:
+    QString originalFileURI;
+
+    QMenu *contextMenu;
 
-    
 };
 
 #endif // IMAGEVIEWER_H
