@@ -36,6 +36,15 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     trayIconAvailable = false;
 
 
+    QString currentIconset = QIcon::themeName();
+    qDebug() << "System iconset:" << currentIconset;
+    qDebug() << "Icon theme search paths:" << QIcon::themeSearchPaths();
+    if (currentIconset.isEmpty() || currentIconset == "hicolor")
+    {
+        qDebug() << ">> No system iconset (or hicolor) configured; trying to use Oxygen";
+        QIcon::setThemeName("oxygen"); // VERY TMPFIX
+    }
+
     // Network control
     pumpController = new PumpController();
 
@@ -57,7 +66,27 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     leftLayout->addSpacing(4);
 
     fullNameLabel = new QLabel("[----------]");
+    fullNameLabel->setWordWrap(true);
     leftLayout->addWidget(fullNameLabel);
+
+
+    QFont userDetailsFont;
+    userDetailsFont.setBold(true);
+    userDetailsFont.setItalic(true);
+    userDetailsFont.setPointSize(userDetailsFont.pointSize() - 2);
+
+    userIdLabel = new QLabel();
+    userIdLabel->setWordWrap(true);
+    userIdLabel->setFont(userDetailsFont);
+    leftLayout->addWidget(userIdLabel);
+
+    userDetailsFont.setBold(false);
+    userDetailsFont.setItalic(false);
+
+    userHometownLabel = new QLabel();
+    userHometownLabel->setWordWrap(true);
+    userHometownLabel->setFont(userDetailsFont);
+    leftLayout->addWidget(userHometownLabel);
     leftLayout->addSpacing(8);
 
 
@@ -87,11 +116,13 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
                                 this);
     mainTimelineScrollArea = new QScrollArea();
     mainTimelineScrollArea->setWidget(mainTimeline);  // Make timeline scrollable
-    mainTimelineScrollArea->setWidgetResizable(true); // Seems to work in opposite way
+    mainTimelineScrollArea->setWidgetResizable(true);
     connect(mainTimeline, SIGNAL(scrollToTop()),
             this, SLOT(scrollMainTimelineToTop()));
     connect(mainTimeline, SIGNAL(timelineRendered(int,int)),
             this, SLOT(setTimelineTabTitle(int,int)));
+    connect(mainTimeline, SIGNAL(unreadPostsCountChanged(int,int)),
+            this, SLOT(setTimelineTabTitle(int,int)));
 
     connect(mainTimeline, SIGNAL(timelineRendered(int,int)),
             this, SLOT(notifyTimelineUpdate(int,int)));
@@ -107,6 +138,11 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
             this, SLOT(scrollDirectTimelineToTop()));
     connect(directTimeline, SIGNAL(timelineRendered(int,int)),
             this, SLOT(setTimelineTabTitle(int,int)));
+    connect(directTimeline, SIGNAL(unreadPostsCountChanged(int,int)),
+            this, SLOT(setTimelineTabTitle(int,int)));
+
+    connect(directTimeline, SIGNAL(timelineRendered(int,int)),
+            this, SLOT(notifyTimelineUpdate(int,int)));
 
 
     activityTimeline = new TimeLine(TimeLine::TimelineTypeActivity,
@@ -119,6 +155,11 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
             this, SLOT(scrollActivityTimelineToTop()));
     connect(activityTimeline, SIGNAL(timelineRendered(int,int)),
             this, SLOT(setTimelineTabTitle(int,int)));
+    connect(activityTimeline, SIGNAL(unreadPostsCountChanged(int,int)),
+            this, SLOT(setTimelineTabTitle(int,int)));
+
+    connect(activityTimeline, SIGNAL(timelineRendered(int,int)),
+            this, SLOT(notifyTimelineUpdate(int,int)));
 
 
     favoritesTimeline = new TimeLine(TimeLine::TimelineTypeFavorites,
@@ -131,7 +172,11 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
             this, SLOT(scrollFavoritesTimelineToTop()));
     connect(favoritesTimeline, SIGNAL(timelineRendered(int,int)),
             this, SLOT(setTimelineTabTitle(int,int)));
+    connect(favoritesTimeline, SIGNAL(unreadPostsCountChanged(int,int)),
+            this, SLOT(setTimelineTabTitle(int,int)));
 
+    connect(favoritesTimeline, SIGNAL(timelineRendered(int,int)),
+            this, SLOT(notifyTimelineUpdate(int,int)));
 
 
     // The contact list has its own tabs with its own scroll areas
@@ -544,7 +589,7 @@ void MainWindow::createMenus()
 
 
     settingsAccount = new QAction(QIcon::fromTheme("dialog-password"),
-                                  tr("&Account Details"), this);
+                                  tr("&Account"), this);
     connect(settingsAccount, SIGNAL(triggered()),
             accountDialog, SLOT(show()));
     settingsMenu->addAction(settingsAccount);
@@ -606,13 +651,13 @@ void MainWindow::createMenus()
  */
 void MainWindow::createTrayIcon()
 {
-    trayIcon = new QSystemTrayIcon(QIcon::fromTheme("dianara",
-                                                    QIcon(":/icon/32x32/dianara.png")), this);
+    trayIcon = new QSystemTrayIcon(this);
 
     if (trayIcon->isSystemTrayAvailable())
     {
         trayIconAvailable = true;
 
+        this->setTrayIconPixmap(); // Set icon for "no unread messages" initially
         this->setTitleAndTrayInfo(this->tabWidget->currentIndex());
 
 
@@ -638,6 +683,35 @@ void MainWindow::createTrayIcon()
 }
 
 
+/*
+ * Set the tray icon's pixmap, with number of unread messages, or nothing
+ *
+ */
+void MainWindow::setTrayIconPixmap(int count)
+{
+    QPixmap iconPixmap = QIcon::fromTheme("dianara",
+                                          QIcon(":/icon/32x32/dianara.png")).pixmap(32, 32);
+
+    // Paint the number of unread messages on top of the pixmap, if != 0
+    if (count > 0)
+    {
+        QFont font;
+        font.setPointSize(font.pointSize() + 1);
+        font.setBold(true);
+
+        QPainter painter(&iconPixmap);
+        painter.setPen(QColor(Qt::white));
+        painter.setFont(font);
+        painter.drawText(0, 0,
+                         30, 30, // Not 32, 32, to have some margin
+                         Qt::AlignRight | Qt::AlignBottom,
+                         QString("%1").arg(count));
+    }
+
+    this->trayIcon->setIcon(iconPixmap);
+}
+
+
 
 /*
  * Load general program settings and state: size, position...
@@ -771,8 +845,10 @@ void MainWindow::updateUserID(QString newUserID)
 
     this->pumpController->setUserCredentials(userID);
 
-    // Remove current user's name and avatar
+    // Remove current user's name, id and avatar
     this->fullNameLabel->setText("--");
+    this->userIdLabel->setText("_@_");
+    this->userHometownLabel->setText("--");
     avatarIconLabel->setPixmap(QPixmap(":/images/no-avatar.png")
                              .scaled(64, 64,
                                      Qt::KeepAspectRatio,
@@ -884,8 +960,19 @@ void MainWindow::showTrayFallbackMessage(QString message)
 void MainWindow::updateProfileData(QString avatarURL, QString fullName,
                                    QString hometown, QString bio)
 {
+    if (!bio.isEmpty())
+    {
+        bio.prepend("<b></b>"); // make it rich text, so it gets wordwrap
+        bio.replace("\n", "<br>"); // HTML newlines
+    }
     this->fullNameLabel->setText(fullName);
+    this->fullNameLabel->setToolTip(bio);
+    this->userIdLabel->setText(this->userID);
+    this->userIdLabel->setToolTip(bio);
+    this->userHometownLabel->setText(hometown);
+    this->userHometownLabel->setToolTip(bio);
     qDebug() << "Updated profile data from server:" << fullName << " @" << hometown;
+
     this->avatarURL = avatarURL;
     qDebug() << "Own avatar URL:" << avatarURL;
 
@@ -906,6 +993,7 @@ void MainWindow::updateProfileData(QString avatarURL, QString fullName,
     {
         pumpController->getAvatar(avatarURL);
     }
+    this->avatarIconLabel->setToolTip(bio);
 
 
     // Fill/update this info in the profile editor too
@@ -1001,7 +1089,9 @@ void MainWindow::notifyTimelineUpdate(int timelineType, int newPostCount)
         this->setStatusBarMessage(tr("Timeline updated.") + " " + newPostsString);
 
 
-        if (this->showNotifications != 2) // If some type of notifications are enabled
+        // If some type of notifications are enabled, and only for main timeline
+        if (this->showNotifications != FDNotifications::NoNotifications
+           && timelineType == TimeLine::TimelineTypeMain)
         {
             this->fdNotifier->showMessage(tr("Timeline updated at %1.").arg(QTime::currentTime().toString())
                                         + " " + newPostsString);
@@ -1064,7 +1154,11 @@ void MainWindow::setTimelineTabTitle(int timelineType, int newPostCount)
     // If the updated tab is the current one, set also window title, etc.
     if (updatedTab == tabWidget->currentIndex())
     {
-        this->setTitleAndTrayInfo(updatedTab);
+        if (trayIconAvailable)
+        {
+            this->setTrayIconPixmap(newPostCount);
+            this->setTitleAndTrayInfo(updatedTab);
+        }
     }
 }
 
@@ -1081,15 +1175,28 @@ void MainWindow::setTitleAndTrayInfo(int currentTab)
     currentTabTitle = this->tabWidget->tabText(currentTab);
     currentTabTitle.remove("&"); // Remove accelators
 
-    QString title = "Dianara - "
-                  + currentTabTitle
-                  + " - "
-                  + (userID.isEmpty() ?
-                     tr("[ Your Pump.io account is not configured ]") : userID);
+    QString title = currentTabTitle
+                  + " - Dianara";
+
+    /*  Not sure if I like it
+
+    if (currentTabTitle.endsWith(")")) // kinda TMP
+    {
+        title.prepend("* ");
+    }
+    */
 
     this->setWindowTitle(title);
     if (trayIconAvailable)
     {
+        title = "<b>Dianara</b>: "
+              + currentTabTitle
+              + "<br><br>["
+              + (userID.isEmpty() ?
+                        tr("Your Pump.io account is not configured") : userID)
+              + "]";
+
+
         this->trayIcon->setToolTip(title);
     }
 }
@@ -1217,7 +1324,7 @@ void MainWindow::visitWebSite()
 void MainWindow::aboutDianara()
 {
     QMessageBox::about(this, tr("About Dianara"),
-                       "<b>Dianara v0.8</b><br>"
+                       "<b>Dianara v0.9</b><br>"
                        "Copyright 2012-2013  JanKusanagi<br><br>"
                        "<a href=\"http://jancoding.wordpress.com/dianara\">"
                        "http://jancoding.wordpress.com/dianara</a><br><br>"
