@@ -21,7 +21,7 @@
 #ifndef MINORFEED_H
 #define MINORFEED_H
 
-#include <QTextBrowser>
+#include <QFrame>
 #include <QVBoxLayout>
 #include <QLabel>
 #include <QFont>
@@ -30,17 +30,19 @@
 #include <QDebug>
 
 #include "pumpcontroller.h"
-#include "timestamp.h"
 #include "mischelpers.h"
+#include "minorfeeditem.h"
 
 
-class MinorFeed : public QTextBrowser
+class MinorFeed : public QFrame
 {
     Q_OBJECT
 
 public:
     explicit MinorFeed(PumpController *pumpController,
                        QWidget *parent = 0);
+    ~MinorFeed();
+    void clearContents();
 
 signals:
 
@@ -51,8 +53,10 @@ public slots:
 
 
 private:
-    PumpController *pController;
+    QList<MinorFeedItem *> itemsInFeed;
 
+    QVBoxLayout *mainLayout;
+    PumpController *pController;
 };
 
 #endif // MINORFEED_H
