@@ -124,3 +124,30 @@ QStringList MiscHelpers::htmlWithReplacedImages(QString originalHtml)
 
     return imageList;
 }
+
+
+/*
+ * Basic cleanup of HTML stuff
+ *
+ */
+QString MiscHelpers::cleanHtml(QString originalHtml)
+{
+    QString cleanHtml = originalHtml;
+
+
+    cleanHtml.replace("\n", " ");  // Remove line breaks, as that results in server error 500
+
+
+    QRegExp headRE("<html><head>.*</head>");
+    headRE.setMinimal(true);
+
+    cleanHtml.remove(headRE);
+    cleanHtml.remove("</body></html>");
+
+    QRegExp bodyRE("<body style.*>");
+    bodyRE.setMinimal(true);
+
+    cleanHtml.remove(bodyRE);
+
+    return cleanHtml;
+}
