@@ -133,7 +133,7 @@ void FDNotifications::showMessage(QString message)
     {
         arguments << QString("dialog-information");   // app_icon otherwise
     }
-    arguments << QString("Dianara notification");
+    arguments << QString(tr("Dianara Notification"));
     arguments << message;
     arguments << QStringList();        // actions
     arguments << QVariantMap();        // hints
