@@ -29,9 +29,6 @@ Post::Post(PumpController *pumpController,
 {
     this->pController = pumpController;
 
-    this->setFrameStyle(QFrame::Box | QFrame::Raised);
-    //this->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken); // alternative?
-
     this->setSizePolicy(QSizePolicy::MinimumExpanding,
                         QSizePolicy::Maximum);
 
@@ -42,6 +39,11 @@ Post::Post(PumpController *pumpController,
     this->postID = postData.value("id");
     this->postType = postData.value("postType");
 
+    postIsUnread = false;
+    if (postData.value("postIsNew") == "true")
+    {
+        this->postIsUnread = true;
+    }
 
     QFont detailsFont;
     detailsFont.setPointSize(detailsFont.pointSize() - 2); // FIXME: check size first
@@ -53,24 +55,33 @@ Post::Post(PumpController *pumpController,
     postIsSharedLabel = new QLabel();
     if (postData.value("postIsShared") == "true")
     {
-        postIsSharedLabel->setText(tr("Via %1").arg(postData.value("sharedBy")));
+        QString sharedBy = postData.value("sharedBy");
+        if (sharedBy.length() > 16)   // very TMP optimization of LOOONG names / FIXME
+        {
+            sharedBy.replace("@", "@ ");
+            sharedBy.replace(".", ". ");
+        }
+        postIsSharedLabel->setText(QString::fromUtf8("\342\231\272  ") // Recycling symbol
+                                   + tr("Via %1").arg(sharedBy));
         postIsSharedLabel->setWordWrap(true);
         postIsSharedLabel->setAlignment(Qt::AlignCenter);
         postIsSharedLabel->setFont(detailsFont);
-        postIsSharedLabel->setForegroundRole(QPalette::Background);
-        postIsSharedLabel->setBackgroundRole(QPalette::Foreground);
+        postIsSharedLabel->setForegroundRole(QPalette::Text);
+        postIsSharedLabel->setBackgroundRole(QPalette::Base);
         postIsSharedLabel->setAutoFillBackground(true);
-        postIsSharedLabel->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
+        postIsSharedLabel->setFrameStyle(QFrame::Panel | QFrame::Raised);
+        postIsSharedLabel->setToolTip(postData.value("sharedById"));
         leftColumnLayout->addWidget(postIsSharedLabel);
     }
-    leftColumnLayout->addSpacing(4);
+    leftColumnLayout->addSpacing(2);
 
 
-    QString authorTooltipInfo = postData.value("authorId");
+    QString authorTooltipInfo = "<b>" + postData.value("authorId") + "</b>";
     authorTooltipInfo.append("<br>");
     if (!postData.value("authorHometown").isEmpty())
     {
-        authorTooltipInfo.append(tr("Hometown") + ": " + postData.value("authorHometown"));
+        authorTooltipInfo.append("<br>");
+        authorTooltipInfo.append("<b>" + tr("Hometown") + "</b>: " + postData.value("authorHometown"));
         authorTooltipInfo.append("<br>");
     }
     if (!postData.value("authorBio").isEmpty())
@@ -78,15 +89,19 @@ Post::Post(PumpController *pumpController,
         authorTooltipInfo.append("<br>");
         authorTooltipInfo.append(postData.value("authorBio"));
     }
+    authorTooltipInfo.replace("\n", "<br>"); // HTML newlines
 
     if (postData.value("authorId") == pController->currentUserID())
     {
         postIsOwn = true;
         qDebug() << "post is our own!";
+
+        this->setFrameStyle(QFrame::Panel | QFrame::Sunken);
     }
     else
     {
         postIsOwn = false;
+        this->setFrameStyle(QFrame::Box | QFrame::Raised);
     }
 
     // Author avatar
@@ -126,15 +141,26 @@ Post::Post(PumpController *pumpController,
     authorFont.setPointSize(authorFont.pointSize()-1);
 
     // Author name
-    postAuthorName = new QLabel(postData.value("author"));
-    postAuthorName->setTextFormat(Qt::PlainText);
-    postAuthorName->setWordWrap(true);
-    postAuthorName->setAlignment(Qt::AlignLeft | Qt::AlignTop);
-    postAuthorName->setFont(authorFont);
-    postAuthorName->setToolTip(authorTooltipInfo);
+    QString authorName = postData.value("author");
+    if (authorName.isEmpty()) // In case name is empty, use webfinger ID
+    {
+        authorName = postData.value("authorId");
+    }
 
-    leftColumnLayout->addWidget(postAuthorName);
-    leftColumnLayout->addSpacing(4);
+    if (authorName.length() > 16)   // very TMP optimization of LOOONG names / FIXME
+    {
+        authorName.replace("@", "@ ");
+        authorName.replace(".", ". ");
+    }
+    postAuthorNameLabel = new QLabel(authorName);
+    postAuthorNameLabel->setTextFormat(Qt::PlainText);
+    postAuthorNameLabel->setWordWrap(true);
+    postAuthorNameLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+    postAuthorNameLabel->setFont(authorFont);
+    postAuthorNameLabel->setToolTip(authorTooltipInfo);
+
+    leftColumnLayout->addWidget(postAuthorNameLabel);
+    leftColumnLayout->addSpacing(2);
 
 
 
@@ -146,23 +172,23 @@ Post::Post(PumpController *pumpController,
     createdAt = Timestamp::fuzzyTime(createdAt);
     QString generator = postData.value("generator");
 
-    postCreatedAt = new QLabel(createdAt);
+    postCreatedAtLabel = new QLabel(createdAt);
     if (generator.isEmpty())
     {
-        postCreatedAt->setToolTip(tr("Posted on %1","1=Date")
+        postCreatedAtLabel->setToolTip(tr("Posted on %1","1=Date")
                                   .arg(exactPostTime));
     }
     else
     {
-        postCreatedAt->setToolTip(tr("Posted on %1\n"
+        postCreatedAtLabel->setToolTip(tr("Posted on %1\n"
                                      "using %2",        "1=Date of post, 2=Program used for posting")
                                   .arg(exactPostTime).arg(generator));
     }
-    postCreatedAt->setTextFormat(Qt::PlainText);
-    postCreatedAt->setWordWrap(true);
-    postCreatedAt->setAlignment(Qt::AlignLeft | Qt::AlignTop);
-    postCreatedAt->setFont(detailsFont);
-    leftColumnLayout->addWidget(postCreatedAt);
+    postCreatedAtLabel->setTextFormat(Qt::PlainText);
+    postCreatedAtLabel->setWordWrap(true);
+    postCreatedAtLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+    postCreatedAtLabel->setFont(detailsFont);
+    leftColumnLayout->addWidget(postCreatedAtLabel);
     leftColumnLayout->addSpacing(4);
 
 
@@ -184,7 +210,7 @@ Post::Post(PumpController *pumpController,
     openInBrowserLabel->setFont(detailsFont);
     leftColumnLayout->addWidget(openInBrowserLabel);
 
-    leftColumnLayout->addSpacing(12);
+    leftColumnLayout->addSpacing(8);
 
 
     this->postLikesURL = postData.value("likesURL");
@@ -284,7 +310,9 @@ Post::Post(PumpController *pumpController,
     connect(commentButton, SIGNAL(clicked()),
             this, SLOT(commentOnPost()));
 
-    shareButton = new QPushButton(QIcon::fromTheme("mail-forward"), tr("Share"));
+    // Note: Gwenview includes 'document-share'
+    shareButton = new QPushButton(QIcon::fromTheme("mail-forward"),
+                                  tr("Share"));
     shareButton->setToolTip(tr("Share this post"));
     shareButton->setFlat(true);
     connect(shareButton, SIGNAL(clicked()),
@@ -299,9 +327,12 @@ Post::Post(PumpController *pumpController,
     buttonsLayout->setMargin(0);
     buttonsLayout->setSpacing(0);
     buttonsLayout->addWidget(likeButton,    0, Qt::AlignLeft);
-    buttonsLayout->addWidget(commentButton, 0, Qt::AlignLeft);
-    buttonsLayout->addWidget(shareButton,   0, Qt::AlignLeft);
-    buttonsLayout->addStretch(1); // so the (optional) Delete buttons gets separated
+    if (postType != "comment") // Don't add these buttons if it's a comment
+    {                          // (can happen in the Favorites timeline)
+        buttonsLayout->addWidget(commentButton, 0, Qt::AlignLeft);
+        buttonsLayout->addWidget(shareButton,   0, Qt::AlignLeft);
+    }
+    buttonsLayout->addStretch(1); // so the (optional) Delete button gets separated
 
     if (postIsOwn)
     {
@@ -394,17 +425,8 @@ Post::Post(PumpController *pumpController,
     postText->setMinimumHeight(postMinimumHeight);
 
 
-    // NSFW button, to replace #nsfw posts until clicked
-    nsfwCoveringButton = new QPushButton("This post has been marked Not Safe For Work (#NSFW).\n"
-                                         "Click here to show it.");
-    nsfwCoveringButton->setSizePolicy(QSizePolicy::MinimumExpanding,
-                                      QSizePolicy::MinimumExpanding);
-    connect(nsfwCoveringButton, SIGNAL(clicked()),
-            this, SLOT(showNsfwPost()));
-
-
     /////////////////////////////////////////////////////// Comments block
-    this->commenter = new Commenter(this->pController);
+    this->commenter = new CommenterBlock(this->pController);
     connect(commenter, SIGNAL(commentSent(QString)),
             this, SLOT(sendComment(QString)));
     connect(commenter, SIGNAL(allCommentsRequested()),
@@ -426,16 +448,14 @@ Post::Post(PumpController *pumpController,
     mainLayout->addLayout(leftColumnLayout,  2); // stretch 2/10
     mainLayout->addLayout(rightColumnLayout, 8); // stretch 8/10
 
-    if (postData.value("nsfw") == "true")
-    {
-        postText->hide();
-        mainLayout->addWidget(nsfwCoveringButton, 5);
-    }
-
     mainLayout->setAlignment(Qt::AlignLeft | Qt::AlignTop);
     this->setLayout(mainLayout);
 
 
+    // Set read status
+    this->setPostUnreadStatus();
+
+
     qDebug() << "Post created";
 }
 
@@ -650,6 +670,36 @@ void Post::setSharesLabel(int resharesCount)
 
 
 
+/*
+ * Set or unset the visual hint indicating if the post is unread
+ *
+ */
+void Post::setPostUnreadStatus()
+{
+    if (this->postIsUnread)
+    {
+        postAuthorNameLabel->setAutoFillBackground(true);
+        postAuthorNameLabel->setForegroundRole(QPalette::HighlightedText);
+        postAuthorNameLabel->setBackgroundRole(QPalette::Highlight);
+
+        postCreatedAtLabel->setAutoFillBackground(true);
+        postCreatedAtLabel->setForegroundRole(QPalette::HighlightedText);
+        postCreatedAtLabel->setBackgroundRole(QPalette::Highlight);
+    }
+    else
+    {
+        postAuthorNameLabel->setAutoFillBackground(false);
+        postAuthorNameLabel->setForegroundRole(QPalette::Foreground);
+        postAuthorNameLabel->setBackgroundRole(QPalette::Background);
+
+        postCreatedAtLabel->setAutoFillBackground(false);
+        postCreatedAtLabel->setForegroundRole(QPalette::Foreground);
+        postCreatedAtLabel->setBackgroundRole(QPalette::Background);
+    }
+}
+
+
+
 
 void Post::resizeEvent(QResizeEvent *event)
 {
@@ -664,21 +714,43 @@ void Post::resizeEvent(QResizeEvent *event)
 }
 
 
+/*
+ * On mouse click in any part of the post, set it as read
+ *
+ */
+void Post::mousePressEvent(QMouseEvent *event)
+{
+    if (postIsUnread)
+    {
+        this->postIsUnread = false;
+        emit postRead(); // Inform the Timeline()
+
+        setPostUnreadStatus();
+    }
+
+    event->accept();
+}
+
+
+/*
+ * Ensure we hide the highlighted URL label when the mouse leaves the post
+ *
+ */
+void Post::leaveEvent(QEvent *event)
+{
+    this->highlightedUrlLabel->clear();
+    this->highlightedUrlLabel->hide();
+
+    event->accept();
+}
+
+
 
 ////////////////////////////////////////////////////////////////////////////
 ////////////////////////////////// SLOTS ///////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////
 
 
-void Post::showNsfwPost()
-{
-    this->nsfwCoveringButton->hide();
-    this->postText->show();
-
-    qDebug() << "Shown NSFW post";
-}
-
-
 /*
  * Like (favorite) a post
  *
@@ -763,7 +835,7 @@ void Post::getAllComments()
 void Post::sharePost()
 {
     int confirmation = QMessageBox::question(this, tr("Share post?"),
-                                             tr("Do you want to share %1's post?").arg(this->postAuthorName->text()),
+                                             tr("Do you want to share %1's post?").arg(this->postAuthorNameLabel->text()),
                                              tr("&Yes, share it"), tr("&No"),
                                              "", 1, 1);
 
