@@ -40,13 +40,13 @@
 #include <QDebug>
 
 
-#include "markdown.h"
 #include "timestamp.h"
 #include "mischelpers.h"
 #include "pumpcontroller.h"
-#include "commenter.h"
+#include "commenterblock.h"
 #include "imageviewer.h"
 
+
 class Post : public QFrame
 {
     Q_OBJECT
@@ -72,11 +72,13 @@ public:
     void setShares(QVariantList sharesList);
     void setSharesLabel(int resharesCount);
 
+    void setPostUnreadStatus();
+
 signals:
+    void postRead();
+
 
 public slots:
-    void showNsfwPost();
-
     void likePost(bool like);
     void fixLikeButton(QString state);
     void getAllLikes();
@@ -92,8 +94,11 @@ public slots:
     void openClickedURL(QUrl url);
     void showHighlightedURL(QUrl url);
 
+
 protected:
     virtual void resizeEvent(QResizeEvent *event);
+    virtual void mousePressEvent(QMouseEvent *event);
+    virtual void leaveEvent(QEvent *event);
 
 
 private:
@@ -104,10 +109,11 @@ private:
     QString postID;
     QString postType;
     bool postIsOwn;
+    bool postIsUnread;
 
     QLabel *postAuthorAvatar;
-    QLabel *postAuthorName;
-    QLabel *postCreatedAt;
+    QLabel *postAuthorNameLabel;
+    QLabel *postCreatedAtLabel;
     QLabel *postToLabel;
     QLabel *postCCLabel;
     QLabel *openInBrowserLabel;
@@ -131,8 +137,6 @@ private:
 
     QString postImageTitle;
 
-    QPushButton *nsfwCoveringButton;
-
     QStringList imageList;
     QString postLikesURL;
     QString postCommentsURL;
@@ -140,7 +144,7 @@ private:
 
 
     PumpController *pController;
-    Commenter *commenter;
+    CommenterBlock *commenter;
 };
 
 #endif // POST_H
