@@ -25,6 +25,10 @@ Publisher::Publisher(PumpController *pumpController,
                      QWidget *parent) : QWidget(parent)
 {
     this->pController = pumpController;
+    connect(pController, SIGNAL(postPublished()),
+            this, SLOT(onPublishingOk()));
+    connect(pController, SIGNAL(postPublishingFailed()),
+            this, SLOT(onPublishingFailed()));
 
     this->setFocusPolicy(Qt::StrongFocus); // To keep the publisher from getting focus by accident
 
@@ -52,6 +56,9 @@ Publisher::Publisher(PumpController *pumpController,
     titleLineEdit->setToolTip(tr("Title for the picture"));
     titleLineEdit->hide();
 
+    pictureInfoLabel = new QLabel();
+
+
     selectPictureButton = new QPushButton(QIcon::fromTheme("insert-image"),
                                           tr("Select Picture..."));
     selectPictureButton->setSizePolicy(QSizePolicy::MinimumExpanding,
@@ -63,7 +70,7 @@ Publisher::Publisher(PumpController *pumpController,
 
     // Set default pixmap and "picture not set" message
     this->setEmptyPictureData();
-
+    lastUsedDirectory = QDir::homePath();
 
 
 
@@ -75,9 +82,20 @@ Publisher::Publisher(PumpController *pumpController,
     connect(composerBox, SIGNAL(editingCancelled()), this, SLOT(setMinimumMode()));
 
 
+    // To... menu
+    toPublicAction = new QAction(tr("Public"), this);
+    toPublicAction->setCheckable(true);
+    connect(toPublicAction, SIGNAL(toggled(bool)),
+            this, SLOT(setToPublic(bool)));
+
+    toFollowersAction = new QAction(tr("Followers"), this);
+    toFollowersAction->setCheckable(true);
+    connect(toFollowersAction, SIGNAL(toggled(bool)),
+            this, SLOT(setToFollowers(bool)));
+
     toSelectorMenu = new QMenu("to-menu");
-    toSelectorMenu->addAction(tr("Public"))->setCheckable(true);
-    toSelectorMenu->addAction(tr("Followers"))->setCheckable(true);
+    toSelectorMenu->addAction(toPublicAction);
+    toSelectorMenu->addAction(toFollowersAction);
     toSelectorMenu->addSeparator();
     toSelectorMenu->addAction(tr("More..."), audienceSelectorTo, SLOT(show()));
 
@@ -87,36 +105,56 @@ Publisher::Publisher(PumpController *pumpController,
     toSelectorButton->setMenu(toSelectorMenu);
 
 
+    // CC... menu
+    ccPublicAction = new QAction(tr("Public"), this);
+    ccPublicAction->setCheckable(true);
+    connect(ccPublicAction, SIGNAL(toggled(bool)),
+            this, SLOT(setCCPublic(bool)));
+
+    ccFollowersAction = new QAction(tr("Followers"), this);
+    ccFollowersAction->setCheckable(true);
+    connect(ccFollowersAction, SIGNAL(toggled(bool)),
+            this, SLOT(setCCFollowers(bool)));
 
     ccSelectorMenu = new QMenu("cc-menu");
-    ccSelectorMenu->addAction(tr("Public"))->setCheckable(true);
-    ccSelectorMenu->addAction(tr("Followers"))->setCheckable(true);
+    ccSelectorMenu->addAction(ccPublicAction);
+    ccSelectorMenu->addAction(ccFollowersAction);
     ccSelectorMenu->addSeparator();
     ccSelectorMenu->addAction(tr("More..."), audienceSelectorCC, SLOT(show()));
 
-    // Default = "CC: followers"
-    ccSelectorMenu->actions().at(1)->setChecked(true); // kinda tmp
-
     ccSelectorButton = new QPushButton(QIcon::fromTheme("system-users"),
                                      tr("CC..."));
     ccSelectorButton->setToolTip(tr("Select who will get a copy of this post"));
     ccSelectorButton->setMenu(ccSelectorMenu);
 
 
-    QFont audienceLabelsFont;
+    QFont audienceLabelsFont; // "To" column will be normal, "CC" will be italic
     audienceLabelsFont.setPointSize(audienceLabelsFont.pointSize()-1);
 
     // These will hold the names of the *people* selected for the To or CC fields, if any
     toAudienceLabel = new QLabel();
     toAudienceLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
-    toAudienceLabel->setFont(audienceLabelsFont);
+    toAudienceLabel->setFont(audienceLabelsFont);  // Normal
     toAudienceLabel->setWordWrap(true);
 
     ccAudienceLabel = new QLabel();
     ccAudienceLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
-    ccAudienceLabel->setFont(audienceLabelsFont);
+    audienceLabelsFont.setItalic(true);
+    ccAudienceLabel->setFont(audienceLabelsFont);  // Italic
     ccAudienceLabel->setWordWrap(true);
 
+    // And these will show "public" or "followers" current selection (in bold)
+    toPublicFollowersLabel = new QLabel();
+    toPublicFollowersLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+    audienceLabelsFont.setBold(true);
+    audienceLabelsFont.setItalic(false);
+    toPublicFollowersLabel->setFont(audienceLabelsFont);  // Bold
+
+    ccPublicFollowersLabel = new QLabel();
+    ccPublicFollowersLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+    audienceLabelsFont.setItalic(true);
+    ccPublicFollowersLabel->setFont(audienceLabelsFont);  // Bold + italic
+
 
     // To set the 'picture mode'
     pictureButton = new QPushButton(QIcon::fromTheme("camera-photo"),
@@ -126,6 +164,15 @@ Publisher::Publisher(PumpController *pumpController,
             this, SLOT(setPictureMode()));
 
 
+    // Sending status info label
+    statusInfoLabel = new QLabel();
+    statusInfoLabel->setAlignment(Qt::AlignCenter);
+    statusInfoLabel->setWordWrap(true);
+    audienceLabelsFont.setBold(false);
+    audienceLabelsFont.setItalic(false);
+    statusInfoLabel->setFont(audienceLabelsFont);
+
+
     // To send the post
     postButton = new QPushButton(QIcon::fromTheme("mail-send"),
                                   tr("Post"));
@@ -133,10 +180,17 @@ Publisher::Publisher(PumpController *pumpController,
     connect(postButton, SIGNAL(clicked()),
             this, SLOT(sendPost()));
 
+    cancelButton = new QPushButton(QIcon::fromTheme("dialog-cancel"),
+                                   tr("Cancel"));
+    cancelButton->setToolTip(tr("Cancel the post"));
+    connect(cancelButton, SIGNAL(clicked()),
+            composerBox, SLOT(cancelPost()));
+
 
     // Now the layout, starting with "picture mode" stuff
     mainLayout->addWidget(pictureLabel,        0, 0, 2, 3);
     mainLayout->addWidget(titleLineEdit,       0, 3, 1, 3, Qt::AlignTop);
+    mainLayout->addWidget(pictureInfoLabel,    1, 3, 1, 1, Qt::AlignBottom);
     mainLayout->addWidget(selectPictureButton, 1, 4, 1, 2, Qt::AlignBottom | Qt::AlignRight);
 
     // The rest depends on the publisher type
@@ -171,16 +225,25 @@ Publisher::Publisher(PumpController *pumpController,
         mainLayout->addWidget(toSelectorButton, 5, 0, 1, 1, Qt::AlignLeft);
         mainLayout->addWidget(ccSelectorButton, 5, 1, 1, 1, Qt::AlignLeft);
         mainLayout->addWidget(pictureButton,    5, 3, 1, 1, Qt::AlignLeft);
-        mainLayout->addWidget(postButton,       5, 5, 1, 1, Qt::AlignRight);
+        mainLayout->addWidget(postButton,       5, 5, 1, 1);
 
         break;
 
     }
 
-    // And finally, the 2 labels holding people's names, if any
+    // The 2 labels holding people's names, if any
     mainLayout->addWidget(toAudienceLabel, 6, 0, 1, 1);
     mainLayout->addWidget(ccAudienceLabel, 6, 1, 1, 1);
 
+    // The 2 labels holding "public/followers", if selected
+    mainLayout->addWidget(toPublicFollowersLabel, 7, 0, 1, 1);
+    mainLayout->addWidget(ccPublicFollowersLabel, 7, 1, 1, 1);
+
+    // the status info label
+    mainLayout->addWidget(statusInfoLabel, 5, 4, 2, 1, Qt::AlignCenter);
+
+    // The "cancel" button
+    mainLayout->addWidget(cancelButton,    7, 5, 1, 1);
 
     this->setLayout(mainLayout);
 
@@ -216,8 +279,6 @@ void Publisher::setDefaultPublicPosting(bool defaultPublicPosts)
  */
 void Publisher::setEmptyPictureData()
 {
-    titleLineEdit->clear();
-
     pictureLabel->setPixmap(QIcon::fromTheme("image-x-generic")
                             .pixmap(200, 150)
                             .scaled(200, 150,
@@ -225,12 +286,41 @@ void Publisher::setEmptyPictureData()
                                     Qt::SmoothTransformation));
     pictureLabel->setToolTip(tr("Picture not set"));
 
+    titleLineEdit->clear();
+    pictureInfoLabel->clear();
 
     this->pictureFilename.clear();
     this->pictureContentType.clear();
 }
 
 
+void Publisher::updatePublicFollowersLabels()
+{
+    QString toString;
+    if (toPublicAction->isChecked())
+    {
+        toString.append("+" + tr("Public") + "\n");
+    }
+    if (toFollowersAction->isChecked())
+    {
+        toString.append("+" + tr("Followers") + "\n");
+    }
+    toPublicFollowersLabel->setText(toString);
+
+
+    QString ccString;
+    if (ccPublicAction->isChecked())
+    {
+        ccString.append("+" + tr("Public") + "\n");
+    }
+    if (ccFollowersAction->isChecked())
+    {
+        ccString.append("+" + tr("Followers") + "\n");
+    }
+    ccPublicFollowersLabel->setText(ccString);
+}
+
+
 
 /*
  * Create a key:value map, listing who will receive a post, like:
@@ -246,7 +336,7 @@ QMap<QString,QString> Publisher::getAudienceMap()
 
 
     // To: Public is checked
-    if (toSelectorMenu->actions().at(0)->isChecked())
+    if (toPublicAction->isChecked())
     {
         audienceMap.insertMulti("to|collection", "http://activityschema.org/collection/public");
     }
@@ -258,14 +348,14 @@ QMap<QString,QString> Publisher::getAudienceMap()
     }
 
     // To: Followers is checked
-    if (toSelectorMenu->actions().at(1)->isChecked())
+    if (toFollowersAction->isChecked())
     {
         audienceMap.insertMulti("to|collection", this->pController->currentFollowersURL());
     }
 
 
     // CC: Public is checked
-    if (ccSelectorMenu->actions().at(0)->isChecked())
+    if (ccPublicAction->isChecked())
     {
         audienceMap.insertMulti("cc|collection", "http://activityschema.org/collection/public");
     }
@@ -278,8 +368,7 @@ QMap<QString,QString> Publisher::getAudienceMap()
 
 
     // Last check: if CC: Followers is checked, or nothing is, add Followers
-    if (ccSelectorMenu->actions().at(1)->isChecked()
-     || audienceMap.isEmpty())
+    if (ccFollowersAction->isChecked() || audienceMap.isEmpty())
     {
         audienceMap.insertMulti("cc|collection", this->pController->currentFollowersURL());
     }
@@ -301,24 +390,15 @@ void Publisher::setMinimumMode()
     qDebug() << "setting Publisher to minimum mode";
 
     this->postButton->setFocus(); // give focus to button,
-                                   // in case user shared with Ctrl+Enter
+                                  // in case user shared with Ctrl+Enter
 
     // disable possible scrollbars
     this->composerBox->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     this->composerBox->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
     // ~1 row
-    this->setMaximumHeight((this->composerBox->fontInfo().pixelSize()+4) * 3);
+    this->setMaximumHeight((this->composerBox->fontInfo().pixelSize() + 6) * 3);
 
-    this->toSelectorButton->hide();
-    this->ccSelectorButton->hide();
-
-    // Check "public" if "public posts" is set in the preferences
-    this->toSelectorMenu->actions().at(0)->setChecked(this->defaultPublicPosting);
-    this->toSelectorMenu->actions().at(1)->setChecked(false);
-
-    this->ccSelectorMenu->actions().at(0)->setChecked(false);
-    this->ccSelectorMenu->actions().at(1)->setChecked(true);  // CC: Followers by default
 
     this->audienceSelectorTo->resetLists();
     this->audienceSelectorCC->resetLists();
@@ -327,17 +407,39 @@ void Publisher::setMinimumMode()
     toAudienceLabel->repaint(); // Avoid a flicker-like effect later
     toAudienceLabel->clear();
     toAudienceLabel->hide();
+
+    this->toAddressStringList.clear();
+
+    toPublicFollowersLabel->hide();
+
+
     this->ccAudienceLabel->setText("...");
     ccAudienceLabel->repaint();
     ccAudienceLabel->clear();
     ccAudienceLabel->hide();
 
-    this->toAddressStringList.clear();
     this->ccAddressStringList.clear();
 
+    ccPublicFollowersLabel->hide();
+
+
+    this->toSelectorButton->hide();
+    this->ccSelectorButton->hide();
+
+    // Check "public" if "public posts" is set in the preferences
+    toPublicAction->setChecked(this->defaultPublicPosting);
+    toFollowersAction->setChecked(false);
+
+    ccPublicAction->setChecked(false);
+    ccFollowersAction->setChecked(true);  // CC: Followers by default
+
 
     this->pictureButton->hide();
+    statusInfoLabel->clear();
+    this->statusInfoLabel->hide();
     this->postButton->hide();
+    this->cancelButton->hide();
+
 
     // Clear formatting options like bold, italic and underline
     this->composerBox->setCurrentCharFormat(QTextCharFormat());
@@ -345,6 +447,7 @@ void Publisher::setMinimumMode()
     // Hide "picture mode" controls
     this->pictureLabel->hide();
     this->titleLineEdit->hide();
+    this->pictureInfoLabel->hide();
     this->selectPictureButton->hide();
 
     this->setEmptyPictureData();
@@ -365,7 +468,10 @@ void Publisher::setFullMode()
     this->ccSelectorButton->show();
 
     this->toAudienceLabel->show();
+    this->toPublicFollowersLabel->show();
+
     this->ccAudienceLabel->show();
+    this->ccPublicFollowersLabel->show();
 
 
     // Avoid re-enabling the picture button when re-focusing publisher, but still in picture mode...
@@ -374,7 +480,9 @@ void Publisher::setFullMode()
         this->pictureButton->setEnabled(true); // If it wasn't hidden, don't re-enable
     }
     this->pictureButton->show();
+    this->statusInfoLabel->show();
     this->postButton->show();
+    this->cancelButton->show();
 
     this->composerBox->setFocus(); // In case user used menu or shortcut
                                    // instead of clicking on it
@@ -390,12 +498,85 @@ void Publisher::setPictureMode()
     this->pictureLabel->show();
 
     this->titleLineEdit->show();
+    this->pictureInfoLabel->show();
     this->selectPictureButton->show();
 
     this->findPictureFile(); // Show the open file dialog directly
 }
 
 
+/*
+ * After the post is confirmed to have been received by the server
+ * re-enable publisher, clear text, etc.
+ *
+ */
+void Publisher::onPublishingOk()
+{
+    this->statusInfoLabel->clear();
+    this->setEnabled(true);
+
+    this->composerBox->erase();
+
+    // Done composing message, hide buttons until we get focus again
+    setMinimumMode();
+}
+
+/*
+ * If there was a HTTP error while posting...
+ *
+ */
+void Publisher::onPublishingFailed()
+{
+    qDebug() << "Posting failed, re-enabling Publisher";
+    this->statusInfoLabel->setText(tr("Posting failed.\nTry again."));
+    this->setEnabled(true);
+    this->composerBox->setFocus();
+}
+
+
+/*
+ * These are called when selecting Public or Followers in the menus
+ *
+ * When selecting "CC: Followers", "To: Followers" gets unselected, etc.
+ *
+ */
+void Publisher::setToPublic(bool activated)
+{
+    if (activated)
+    {
+        ccPublicAction->setChecked(false);     // Unselect "CC: Public"
+    }
+    updatePublicFollowersLabels();
+}
+
+void Publisher::setToFollowers(bool activated)
+{
+    if (activated)
+    {
+        ccFollowersAction->setChecked(false);  // Unselect "CC: Followers"
+    }
+    updatePublicFollowersLabels();
+}
+
+void Publisher::setCCPublic(bool activated)
+{
+    if (activated)
+    {
+        toPublicAction->setChecked(false);     // Unselect "To: Public"
+    }
+    updatePublicFollowersLabels();
+}
+
+void Publisher::setCCFollowers(bool activated)
+{
+    if (activated)
+    {
+        toFollowersAction->setChecked(false);  // Unselect "To: Followers"
+    }
+    updatePublicFollowersLabels();
+}
+
+
 
 
 void Publisher::findPictureFile()
@@ -403,7 +584,7 @@ void Publisher::findPictureFile()
     QString filename;
 
     filename = QFileDialog::getOpenFileName(this, tr("Select one image"),
-                                            QDir::homePath(),
+                                            this->lastUsedDirectory,
                                             tr("Image files") + "(*.png *.jpg *.jpeg *.gif);;"
                                             + tr("All files") + " (*)");
 
@@ -412,18 +593,41 @@ void Publisher::findPictureFile()
         qDebug() << "Selected" << filename << "for upload";
         this->pictureLabel->setToolTip(filename);
 
-        QString imageFormat = QString::fromLocal8Bit(QImageReader::imageFormat(filename));
-        qDebug() << "Image format:" << imageFormat;
+        QString formatString = QString::fromLocal8Bit(QImageReader::imageFormat(filename));
+        qDebug() << "Image format:" << formatString;
 
-        if (imageFormat == "png" || imageFormat == "jpeg" || imageFormat == "gif")
+        if (formatString == "png" || formatString == "jpeg" || formatString == "gif")
         {
-            this->pictureContentType = "image/" + imageFormat;
-            qDebug() << pictureContentType;
+            this->pictureContentType = "image/" + formatString;
             this->pictureFilename = filename;
 
-            this->pictureLabel->setPixmap(QPixmap(filename).scaled(200, 150,
-                                                                   Qt::IgnoreAspectRatio,
-                                                                   Qt::SmoothTransformation));
+            QPixmap imagePixmap = QPixmap(filename);
+            this->pictureLabel->setPixmap(imagePixmap.scaled(200, 150,
+                                                             Qt::IgnoreAspectRatio,
+                                                             Qt::SmoothTransformation));
+
+            QFileInfo fileInfo(filename);
+            int filesize = fileInfo.size();
+            this->lastUsedDirectory = fileInfo.path();
+            qDebug() << "last used directory:" << lastUsedDirectory;
+            QString sizeUnit = tr("bytes");
+            if (filesize > 1024) // if > 1024 bytes, transform to KiB
+            {
+                filesize /= 1024;
+                sizeUnit = "KiB";
+            }
+            if (filesize > 1024) // if > 1024 KiB, transform to MiB
+            {
+                filesize /= 1024;
+                sizeUnit = "MiB";
+            }
+            pictureInfoLabel->setText(QString("<b>" +tr("Resolution") + ":</b> %1x%2"
+                                      "<br>"
+                                      "<b>" + tr("Size") + ":</b> %3 %4")
+                                      .arg(imagePixmap.width())
+                                      .arg(imagePixmap.height())
+                                      .arg(filesize)
+                                      .arg(sizeUnit));
         }
         else
         {
@@ -437,8 +641,6 @@ void Publisher::findPictureFile()
             this->pictureContentType.clear();
             this->pictureFilename.clear();
         }
-
-
     }
 }
 
@@ -507,6 +709,11 @@ void Publisher::sendPost()
         QMap<QString,QString> audienceMap = this->getAudienceMap();
 
 
+        this->statusInfoLabel->setText(tr("Posting..."));
+        // Don't erase just yet!! Just disable until we get "200 OK" from the server.
+        this->setDisabled(true);
+
+
         if (this->pictureFilename.isEmpty())
         {
             this->pController->postNote(audienceMap, cleanHtml);
@@ -519,12 +726,6 @@ void Publisher::sendPost()
                                          pictureFilename,
                                          pictureContentType);
         }
-
-
-        this->composerBox->erase();
-
-        // Done composing message, hide buttons until we get focus again
-        setMinimumMode();
     }
     else
     {
