@@ -51,6 +51,7 @@ public:
 
     void setDefaultPublicPosting(bool defaultPublicPosts);
     void setEmptyPictureData();
+    void updatePublicFollowersLabels();
 
     QMap<QString,QString> getAudienceMap();
 
@@ -63,6 +64,14 @@ public slots:
     void setFullMode();
     void setPictureMode();
 
+    void onPublishingOk();
+    void onPublishingFailed();
+
+    void setToPublic(bool activated);
+    void setToFollowers(bool activated);
+    void setCCPublic(bool activated);
+    void setCCFollowers(bool activated);
+
     void findPictureFile();
 
     void updateToCcFields(QString selectorType,
@@ -73,22 +82,29 @@ public slots:
 private:
     QGridLayout *mainLayout;
 
-    QLineEdit *titleLineEdit;
     QLabel *pictureLabel;
+    QLineEdit *titleLineEdit;
+    QLabel *pictureInfoLabel;
     QPushButton *selectPictureButton;
 
     Composer *composerBox;
 
-
-    QMenu *toSelectorMenu;
     QPushButton *toSelectorButton;
-    QMenu *ccSelectorMenu;
+    QAction *toPublicAction;
+    QAction *toFollowersAction;
+    QMenu *toSelectorMenu;
+
     QPushButton *ccSelectorButton;
+    QAction *ccPublicAction;
+    QAction *ccFollowersAction;
+    QMenu *ccSelectorMenu;
 
     AudienceSelector *audienceSelectorTo;
     AudienceSelector *audienceSelectorCC;
 
+    QLabel *toPublicFollowersLabel;
     QLabel *toAudienceLabel;
+    QLabel *ccPublicFollowersLabel;
     QLabel *ccAudienceLabel;
 
     QStringList toAddressStringList;
@@ -98,11 +114,15 @@ private:
 
 
     QPushButton *pictureButton;
+    QLabel *statusInfoLabel;
+
     QPushButton *postButton;
+    QPushButton *cancelButton;
 
 
     QString pictureFilename;
     QString pictureContentType;
+    QString lastUsedDirectory;
 
 
     PumpController *pController;
