@@ -22,7 +22,7 @@
 
 PumpController::PumpController(QObject *parent) :  QObject(parent)
 {
-    this->userAgentString = "Dianara/0.8";
+    this->userAgentString = "Dianara/0.9";
 
     this->postsPerPage = 20;
     this->updatesToTimelineBlocked = false;
@@ -42,7 +42,7 @@ PumpController::PumpController(QObject *parent) :  QObject(parent)
 
     if (isApplicationAuthorized)
     {
-        qDebug() << "Dianara is already authorized for user ID:" << settings.value("userID");
+        qDebug() << "Dianara is already authorized for user ID:" << settings.value("userID").toString();
 
         this->token = settings.value("token", "").toString().toLocal8Bit();
         this->tokenSecret = settings.value("tokenSecret", "").toString().toLocal8Bit();
@@ -159,6 +159,11 @@ QString PumpController::currentUserID()
     return this->userID;
 }
 
+QString PumpController::currentUsername()
+{
+    return this->userName;
+}
+
 QString PumpController::currentFollowersURL()
 {
     return this->userFollowersURL;
@@ -218,8 +223,7 @@ void PumpController::updateUserProfile(QString avatarURL, QString fullName,
 
 
     QJson::Serializer serializer;
-    bool ok;
-    QByteArray data = serializer.serialize(jsonVariant, &ok);
+    QByteArray data = serializer.serialize(jsonVariant);
 
 
 
@@ -276,18 +280,19 @@ void PumpController::getContactList(QString listType)
     QString url = "https://" + this->serverURL + "/api/user/" +this->userName + "/" + listType;
 
     QOAuth::ParamMap paramMap;
-    paramMap.insert("count", "100");  // TMP/FIXME get 100 contacts
+    paramMap.insert("count", "200");  // TMP/FIXME get 200 contacts
+                                      // To get more, we need pagination
 
     QNetworkRequest contactListRequest;
     if (listType == "following")
     {
-        emit currentJobChanged(tr("Getting followed list..."));
+        emit currentJobChanged(tr("Getting list of 'Following'..."));
         contactListRequest = this->prepareRequest(url, QOAuth::GET,
                                                   FollowingListRequest, paramMap);
     }
     else
     {
-        emit currentJobChanged(tr("Getting followers list..."));
+        emit currentJobChanged(tr("Getting list of 'Followers'..."));
         contactListRequest = this->prepareRequest(url, QOAuth::GET,
                                                   FollowersListRequest, paramMap);
     }
@@ -566,29 +571,77 @@ void PumpController::uploadFile(QString filename, QString contentType)
 
     QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/uploads";
 
-    /*QOAuth::ParamMap paramMap;
-    paramMap.insert("title", "imgTitle");
-    paramMap.insert("description", "imgDescription");*/
     QNetworkRequest postRequest = this->prepareRequest(url, QOAuth::POST,
                                                        UploadFileRequest,
                                                        QOAuth::ParamMap(),
                                                        contentType);
 
-    /*url.append("?title=imgTitle&description=imgDescription");
-    postRequest.setUrl(QUrl(url));*/
-
     QFile file(filename);
     file.open(QIODevice::ReadOnly);
 
     QByteArray data = file.readAll();
     file.close();
 
-
     nam.post(postRequest, data);
 }
 
 
 
+QList<QVariantList> PumpController::processAudience(QMap<QString, QString> audienceMap)
+{
+    QVariantList jsonVariantTo;
+    QVariantList jsonVariantCC;
+
+    QVariantMap jsonVariantAudienceItem;
+    while (!audienceMap.isEmpty())
+    {
+        jsonVariantAudienceItem.clear();
+
+        if (audienceMap.keys().contains("to|collection")) // To:
+        {
+            QString collectionID = audienceMap.take("to|collection");
+            jsonVariantAudienceItem.insert("objectType", "collection");
+            jsonVariantAudienceItem.insert("id", collectionID);
+
+            jsonVariantTo.append(jsonVariantAudienceItem);
+        }
+        else if (audienceMap.keys().contains("to|person"))
+        {
+            QString personID = audienceMap.take("to|person");
+            jsonVariantAudienceItem.insert("objectType", "person");
+            jsonVariantAudienceItem.insert("id", personID);
+
+            jsonVariantTo.append(jsonVariantAudienceItem);
+        }
+        else if (audienceMap.keys().contains("cc|collection")) // CC:
+        {
+            QString collectionID = audienceMap.take("cc|collection");
+            jsonVariantAudienceItem.insert("objectType", "collection");
+            jsonVariantAudienceItem.insert("id", collectionID);
+
+            jsonVariantCC.append(jsonVariantAudienceItem);
+        }
+        else if (audienceMap.keys().contains("cc|person"))
+        {
+            QString personID = audienceMap.take("cc|person");
+            jsonVariantAudienceItem.insert("objectType", "person");
+            jsonVariantAudienceItem.insert("id", personID);
+
+            jsonVariantCC.append(jsonVariantAudienceItem);
+        }
+    }
+
+
+    QList<QVariantList> jsonVariantToAndCCList;
+    jsonVariantToAndCCList.append(jsonVariantTo);
+    jsonVariantToAndCCList.append(jsonVariantCC);
+
+
+    return jsonVariantToAndCCList;
+}
+
+
+
 /***************************************************************************/
 /***************************************************************************/
 /***************************************************************************/
@@ -611,49 +664,107 @@ void PumpController::requestFinished(QNetworkReply *reply)
     qDebug() << "Size:" << reply->size() << "bytes; URL:" << reply->url();
     qDebug() << "isFinished()?" << reply->isFinished() << "; Request type:" << requestType;
 
+
+    // Special control after sending a post or a comment
+    if (httpCode != 200) // if not OK
+    {
+        if (requestType == PublishPostRequest)
+        {
+            emit postPublishingFailed();
+        }
+        else if (requestType == UpdatePostRequest)
+        {
+            emit postPublishingFailed();  // kinda TMP
+        }
+        else if (requestType == CommentPostRequest)
+        {
+            emit commentPostingFailed();
+        }
+    }
+
+
     switch (httpCode)
     {
     case 503:
-        emit showNotification("Service Unavailable (503)\n" + reply->url().toString());
+        // if not just and image, it's important, so popup notification
+        if (requestType != ImageRequest && requestType != AvatarRequest)
+        {
+            emit showNotification("Service Unavailable (503)\n" + reply->url().toString());
+        }
+        else // just status bar
+        {
+            emit currentJobChanged("Service Unavailable (503): " + reply->url().toString());
+        }
+
         qDebug() << "HTTP 503: Service Unavailable.";
         qDebug() << "Data:  " << reply->readAll();
         return;
 
     case 500:
-        emit showNotification("Internal server error (500)\n" + reply->url().toString());
+        // if not just and image, it's important, so popup notification
+        if (requestType != ImageRequest && requestType != AvatarRequest)
+        {
+            emit showNotification("Internal server error (500)\n" + reply->url().toString());
+        }
+        else
+        {
+            emit currentJobChanged("Internal server error (500): " + reply->url().toString());
+        }
         qDebug() << "HTTP 500: Internal server error.";
         qDebug() << "Data:  " << reply->readAll();
         return;
 
 
+    case 410:
+        emit currentJobChanged("Gone (410): " + reply->url().toString());
+        qDebug() << "HTTP 410: Gone.";
+        qDebug() << "Data:  " << reply->readAll();
+        return;
+
     case 404:
-        emit showNotification("Not Found (404)\n" + reply->url().toString());
+        emit currentJobChanged("Not Found (404): " + reply->url().toString());
         qDebug() << "HTTP 404: Not Found.";
         qDebug() << "Data:  " << reply->readAll();
         return;
 
     case 403:
-        emit showNotification("Forbidden (403)\n" + reply->url().toString());
+        emit currentJobChanged("Forbidden (403): " + reply->url().toString());
         qDebug() << "HTTP 403: Forbidden.";
         qDebug() << "Data:  " << reply->readAll();
         return;
 
 
     case 400:
-        emit showNotification("Bad Request (400)\n" + reply->url().toString());
+        // if not just and image, it's important, so popup notification
+        if (requestType != ImageRequest && requestType != AvatarRequest)
+        {
+            emit showNotification("Bad Request (400)\n" + reply->url().toString());
+        }
+        else
+        {
+            emit currentJobChanged("Bad Request (400): " + reply->url().toString());
+        }
         qDebug() << "HTTP 400: Bad Request.";
         qDebug() << "Data:  " << reply->readAll();
         return;
 
+    case 301:
+        emit currentJobChanged("Moved Permanently (301): " + reply->url().toString());
+        qDebug() << "HTTP 301: Moved Permanently.";
+        qDebug() << "Data:  " << reply->readAll();
+        return;
+
     case 0:
         emit currentJobChanged(tr("Error connecting to %1").arg(reply->url().host()) + ": " + reply->errorString());
+        qDebug() << "Error connecting to" << reply->url().host();
         return;
 
     // Other HTTP codes
     default:
         qDebug() << "Unhandled HTTP error " << httpCode;
-        emit showNotification(tr("Unhandled HTTP error code %1").arg(httpCode)
-                              + "\n" + reply->url().toString());
+        qDebug() << "Data:  " << reply->readAll();
+        emit currentJobChanged(tr("Unhandled HTTP error code %1").arg(httpCode)
+                              + ": " + reply->url().toString());
 
         return;
 
@@ -686,6 +797,8 @@ void PumpController::requestFinished(QNetworkReply *reply)
     }
 
 
+    //////////////////////////////////////////////////////////////////
+
     switch (requestType)
     {
 
@@ -800,6 +913,38 @@ void PumpController::requestFinished(QNetworkReply *reply)
     ////////////////////////////////////
     case PublishPostRequest:
         emit currentJobChanged(tr("Post published successfully."));
+        if (jsonParsedOK && jsonData.size() > 0)
+        {
+            qDebug() << "JSON parsed OK";
+
+            QString objectID = jsonData["object"].toMap()["id"].toString();
+            qDebug() << "Image post ID:" << objectID;
+
+            if (jsonData["object"].toMap()["objectType"].toString() == "image")
+            {
+                // Update the image with title and description
+                this->postImageStepThree(objectID);
+            }
+            else
+            {
+                // Not an image, notify "posted OK"
+                emit postPublished();
+                qDebug() << "Non-image post published correctly";
+            }
+        }
+        else
+        {
+            qDebug() << "Error parsing received JSON data!";
+            qDebug() << "Raw data:" << packetData; // JSON directly
+        }
+
+        break;
+
+
+    ////////////////////////////////////
+    case UpdatePostRequest:
+        qDebug() << "**************** POST UPDATED OK!";
+        emit currentJobChanged(tr("Post updated successfully."));
         emit postPublished();
         break;
 
@@ -807,7 +952,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
     ///////////////////////////////////// If liking a post was requested
     case LikePostRequest:
-        emit currentJobChanged(tr("Post liked successfully."));
+        emit currentJobChanged(tr("Message liked or unliked successfully."));
         emit likeSet();
         break;
 
@@ -900,7 +1045,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
 
     case DeletePostRequest:
-        emit currentJobChanged(tr("Post deleted successfully."));
+        emit currentJobChanged(tr("Message deleted successfully."));
         break;
 
 
@@ -1027,7 +1172,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
         if (jsonParsedOK && jsonData.size() > 0)
         {
             qDebug() << "JSON parsed OK";
-            emit currentJobChanged(tr("File uploaded succesfully. Posting message..."));
+            emit currentJobChanged(tr("File uploaded successfully. Posting message..."));
 
             QString uploadedFileID = jsonData["id"].toString();
             qDebug() << "Uploaded file ID:" << uploadedFileID;
@@ -1123,6 +1268,23 @@ void PumpController::getToken()
 
         qDebug() << "Doing OAuth token stuff...";
 
+        qDebug() << "NOTE: if you see a crash here, you need QCA and its openSSL plugin:";
+        qDebug() << ">> qca2-plugin-openssl, libqca2-plugin-ossl, or similar";
+        qDebug() << "If you compiled Dianara from source, check the INSTALL file carefully";
+
+        QStringList QCAsupportedFeatures = QCA::supportedFeatures();
+        qDebug() << "QCA Supported Features:" << QCAsupportedFeatures;
+
+        if (QCAsupportedFeatures.contains("hmac(sha1)"))
+        {
+            qDebug() << "HMAC-SHA1 support is OK";
+        }
+        else
+        {
+            qDebug() << "Warning, HMAC-SHA1 doesn't seem to be supported!";
+
+            // TMPFIX notify the user properly, etc.
+        }
 
         qoauth->setConsumerKey(this->clientID.toLocal8Bit());
         qoauth->setConsumerSecret(this->clientSecret.toLocal8Bit());
@@ -1153,10 +1315,19 @@ void PumpController::getToken()
             qDebug() << "Token Secret:" << tokenSecret;
 
             QUrl oAuthAuthorizeURL("https://" + this->serverURL + "/oauth/authorize");
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
             oAuthAuthorizeURL.addQueryItem("oauth_token", token);
-
+#else
+            QUrlQuery query;
+            query.addQueryItem("oauth_token", token);
+            oAuthAuthorizeURL.setQuery(query);
+#endif
             QDesktopServices::openUrl(oAuthAuthorizeURL);
 
+            // Send also a signal, so AccountDialog can show the URL in
+            // a label, in case the browser didn't launch
+            emit openingAuthorizeURL(oAuthAuthorizeURL);
+
             // Now, user should enter VERIFIER in AccountDialog to authorize the program
         }
         else
@@ -1298,49 +1469,6 @@ void PumpController::postNote(QMap<QString, QString> audienceMap,
     qDebug() << "Should be posting to:" << audienceMap;
 
 
-    QVariantList jsonVariantTo;
-    QVariantList jsonVariantCC;
-
-    QVariantMap jsonVariantAudienceItem;
-    while (!audienceMap.isEmpty())
-    {
-        jsonVariantAudienceItem.clear();
-
-        if (audienceMap.keys().contains("to|collection")) // To:
-        {
-            QString collectionID = audienceMap.take("to|collection");
-            jsonVariantAudienceItem.insert("objectType", "collection");
-            jsonVariantAudienceItem.insert("id", collectionID);
-
-            jsonVariantTo.append(jsonVariantAudienceItem);
-        }
-        else if (audienceMap.keys().contains("to|person"))
-        {
-            QString personID = audienceMap.take("to|person");
-            jsonVariantAudienceItem.insert("objectType", "person");
-            jsonVariantAudienceItem.insert("id", personID);
-
-            jsonVariantTo.append(jsonVariantAudienceItem);
-        }
-        else if (audienceMap.keys().contains("cc|collection")) // CC:
-        {
-            QString collectionID = audienceMap.take("cc|collection");
-            jsonVariantAudienceItem.insert("objectType", "collection");
-            jsonVariantAudienceItem.insert("id", collectionID);
-
-            jsonVariantCC.append(jsonVariantAudienceItem);
-        }
-        else if (audienceMap.keys().contains("cc|person"))
-        {
-            QString personID = audienceMap.take("cc|person");
-            jsonVariantAudienceItem.insert("objectType", "person");
-            jsonVariantAudienceItem.insert("id", personID);
-
-            jsonVariantCC.append(jsonVariantAudienceItem);
-        }
-    }
-
-
 
     QVariantMap jsonVariantObject;
     jsonVariantObject.insert("objectType", "note");
@@ -1349,14 +1477,18 @@ void PumpController::postNote(QMap<QString, QString> audienceMap,
     QVariantMap jsonVariant;
     jsonVariant.insert("verb", "post");
     jsonVariant.insert("object", jsonVariantObject);
-    jsonVariant.insert("to", jsonVariantTo);
-    jsonVariant.insert("cc", jsonVariantCC);
 
+    QList<QVariantList> audience = processAudience(audienceMap);
+    jsonVariant.insert("to", audience.at(0));
+    jsonVariant.insert("cc", audience.at(1));
 
 
     QJson::Serializer serializer;
-    bool ok;
-    QByteArray data = serializer.serialize(jsonVariant, &ok);
+    // bool ok;
+    // QByteArray data = serializer.serialize(jsonVariant, &ok);
+
+    // This way to make it work with QJSON 0.7.x
+    QByteArray data = serializer.serialize(jsonVariant);
 
 
     qDebug() << "About to POST:" << data;
@@ -1379,9 +1511,10 @@ void PumpController::postImage(QMap<QString, QString> audienceMap,
     qDebug() << "PumpController::postImage()";
     qDebug() << "Uploading" << imageFilename << "with title:" << imageTitle;
 
-    // Store postText and imageTitle, then upload
-    this->imageTitle = imageTitle;
-    this->imageDescription = postText;
+    // Store postText, imageTitle, and audienceMap, then upload
+    this->currentImageTitle = imageTitle;
+    this->currentImageDescription = postText;
+    this->currentAudienceMap = audienceMap;
 
 
     this->uploadFile(imageFilename, contentType);
@@ -1405,17 +1538,17 @@ void PumpController::postImageStepTwo(QString id)
     QVariantMap jsonVariantObject;
     jsonVariantObject.insert("objectType", "image");
     jsonVariantObject.insert("id", id);
-    jsonVariantObject.insert("displayName", this->imageTitle);   // not 'title'
-    jsonVariantObject.insert("content", this->imageDescription); // not 'description' or 'summary'
 
     QVariantMap jsonVariant;
     jsonVariant.insert("verb", "post");
     jsonVariant.insert("object", jsonVariantObject);
 
+    QList<QVariantList> audience = processAudience(this->currentAudienceMap);
+    jsonVariant.insert("to", audience.at(0));
+    jsonVariant.insert("cc", audience.at(1));
 
     QJson::Serializer serializer;
-    bool ok;
-    QByteArray data = serializer.serialize(jsonVariant, &ok);
+    QByteArray data = serializer.serialize(jsonVariant);
 
 
     qDebug() << "About to POST:" << data;
@@ -1424,9 +1557,35 @@ void PumpController::postImageStepTwo(QString id)
 }
 
 
+/*
+ * Workaround for the non-title-non-description issue
+ *
+ * Update the image post with the title and the description
+ *
+ */
+void PumpController::postImageStepThree(QString id)
+{
+    qDebug() << "PumpController::postImageStepThree() post ID:" << id;
+
+    // Using the ID directly as URL
+    QNetworkRequest postRequest = this->prepareRequest(id, QOAuth::PUT, UpdatePostRequest);
+
+    QVariantMap jsonVariant;
+    jsonVariant.insert("verb", "update"); // "post" would suffice
+    jsonVariant.insert("displayName", this->currentImageTitle);
+    jsonVariant.insert("content", this->currentImageDescription);
+
+    QJson::Serializer serializer;
+    QByteArray data = serializer.serialize(jsonVariant);
+
+    qDebug() << "About to PUT:" << data;
+    nam.put(postRequest, data);
+}
+
+
 
 /*
- * Like (favorite) a post, by its URL
+ * Like (favorite) a post, by its ID (URL)
  *
  */
 void PumpController::likePost(QString postID, QString postType, bool like)
@@ -1447,8 +1606,7 @@ void PumpController::likePost(QString postID, QString postType, bool like)
     jsonVariant.insert("object", jsonVariantObject);
 
     QJson::Serializer serializer;
-    bool ok;
-    QByteArray data = serializer.serialize(jsonVariant, &ok);
+    QByteArray data = serializer.serialize(jsonVariant);
 
     qDebug() << "about to POST:" << data;
 
@@ -1481,8 +1639,7 @@ void PumpController::addComment(QString comment, QString postID, QString postTyp
     jsonVariant.insert("object", jsonVariantObject);
 
     QJson::Serializer serializer;
-    bool ok;
-    QByteArray data = serializer.serialize(jsonVariant, &ok);
+    QByteArray data = serializer.serialize(jsonVariant);
 
     qDebug() << "about to POST:" << data;
 
@@ -1510,8 +1667,7 @@ void PumpController::sharePost(QString postID, QString postType)
     jsonVariant.insert("object", jsonVariantObject);
 
     QJson::Serializer serializer;
-    bool ok;
-    QByteArray data = serializer.serialize(jsonVariant, &ok);
+    QByteArray data = serializer.serialize(jsonVariant);
 
     qDebug() << "about to POST:" << data;
 
@@ -1538,8 +1694,7 @@ void PumpController::deletePost(QString postID, QString postType)
     jsonVariant.insert("object", jsonVariantObject);
 
     QJson::Serializer serializer;
-    bool ok;
-    QByteArray data = serializer.serialize(jsonVariant, &ok);
+    QByteArray data = serializer.serialize(jsonVariant);
 
     qDebug() << "about to POST:" << data;
 
@@ -1574,8 +1729,7 @@ void PumpController::followContact(QString address)
     jsonVariant.insert("object", jsonVariantObject);
 
     QJson::Serializer serializer;
-    bool ok;
-    QByteArray data = serializer.serialize(jsonVariant, &ok);
+    QByteArray data = serializer.serialize(jsonVariant);
 
     qDebug() << "about to POST:" << data;
 
@@ -1609,8 +1763,7 @@ void PumpController::unfollowContact(QString address)
     jsonVariant.insert("object", jsonVariantObject);
 
     QJson::Serializer serializer;
-    bool ok;
-    QByteArray data = serializer.serialize(jsonVariant, &ok);
+    QByteArray data = serializer.serialize(jsonVariant);
 
     qDebug() << "about to POST:" << data;
 
