@@ -38,7 +38,7 @@ TimeLine::TimeLine(int timelineType,
     demoPostData.insert("authorBio",      "I am not a real user");
     demoPostData.insert("authorHometown", "Demoville");
     demoPostData.insert("generator",      "Dianara");
-    demoPostData.insert("createdAt",      "2012-01-09T00:00:00Z");
+    demoPostData.insert("createdAt",      "2013-05-01T00:00:00Z");
 
 
     QSettings settings; // FIXME: kinda tmp, until posts have "unread" status, etc.
@@ -48,47 +48,71 @@ TimeLine::TimeLine(int timelineType,
     switch (this->timelineType)
     {
     case TimelineTypeMain:
-        demoPostData.insert("text",   "<h2><b>Welcome to Dianara</b></h2><br>"
+        demoPostData.insert("text",   "<h2><b>" +tr("Welcome to Dianara") + "</b></h2><br>"
 
-                                      "Dianara is a <a href=\"http://pump.io\">pump.io</a> client.<br><br>"
+                                      + tr("Dianara is a <b>pump.io</b> client.")
+                                      + "<br>"
 
-                                      "After you have set up your account in <b>Settings > Account Details</b>,<br>"
-                                      "you can press <b>F5</b> or <i>Update timelines</i> in the menu.<br><br>"
+                                      + tr("If you don't have a Pump account yet, you can get one "
+                                           "at the following address:")
+                                      + "<br>"
+                                        "<a href=\"http://pump.io/tryit.html\">http://pump.io/tryit.html</a>"
+                                        "<br><br>"
 
-                                      "Upon startup, timeline should update automatically.<br><br><br>"
+                                      + tr("First, configure your account from the "
+                                           "<b>Settings - Account</b> menu.")
+                                      + "<br>"
+                                      + tr("After the process is done, your profile "
+                                           "and timelines should update automatically.")
+                                      + "<br><br>"
+
+                                      + tr("Take a moment to look around the menus and "
+                                           "the Configuration window.")
+                                      + "<br><br>"
+
+                                      + tr("There are tooltips everywhere, so if you hover "
+                                           "a button or a text field with your mouse, you'll "
+                                           "probably see some extra information.")
+                                      + "<br><br>"
+
+                                      + "<a href=\"http://jancoding.wordpress.com/dianara\">"
+                                      + tr("Dianara's blog") + "</a><br><br>"
+
+                                        "<a href=\"https://github.com/e14n/pump.io/wiki/FAQ\">"
+                                      + tr("Frequently asked questions about pump.io")
+                                      + "</a>"
+                                      + "<br>");
 
-                                      "<a href=\"http://jancoding.wordpress.com/dianara\">Dianara's blog</a><br>"
-                                      "<br>");
         this->previousNewestPostID = settings.value("previousNewestPostIDMain", "").toString();
         break;
 
 
     case TimelineTypeDirect:
-        demoPostData.insert("text",   "<h2><b>Direct Messages Timeline</b></h2><br>"
+        demoPostData.insert("text",   "<h2><b>" + tr("Direct Messages Timeline") + "</b></h2><br>"
 
-                                      "Here, you'll see posts specifically directed to you."
+                                      + tr("Here, you'll see posts specifically directed to you.")
 
-                                      "<br>");
+                                      + "<br>");
         this->previousNewestPostID = settings.value("previousNewestPostIDDirect", "").toString();
         break;
 
 
     case TimelineTypeActivity:
-        demoPostData.insert("text",   "<h2><b>Activity Timeline</b></h2><br>"
+        demoPostData.insert("text",   "<h2><b>" + tr("Activity Timeline") + "</b></h2><br>"
 
-                                      "You'll see your own posts here."
+                                      + tr("You'll see your own posts here.")
 
-                                      "<br>");
+                                      +"<br>");
         this->previousNewestPostID = settings.value("previousNewestPostIDActivity", "").toString();
         break;
 
 
     case TimelineTypeFavorites:
-        demoPostData.insert("text",   "<h2><b>Favorites Timeline</b></h2><br>"
+        demoPostData.insert("text",   "<h2><b>" + tr("Favorites Timeline") + "</b></h2><br>"
 
-                                      "Posts you've liked."
+                                      + tr("Posts and comments you've liked.")
 
-                                      "<br>");
+                                      + "<br>");
         this->previousNewestPostID = settings.value("previousNewestPostIDFavorites", "").toString();
         break;
 
@@ -107,6 +131,7 @@ TimeLine::TimeLine(int timelineType,
     mainLayout->addWidget(splitter);
 
 
+    this->unreadPostsCount = 0;
     this->timelineOffset = 0;
 
     firstPageButton = new QPushButton(QIcon::fromTheme("go-first"), tr("F&irst Page"));
@@ -128,9 +153,9 @@ TimeLine::TimeLine(int timelineType,
 
     bottomLayout = new QHBoxLayout();
     bottomLayout->addWidget(firstPageButton);
-    bottomLayout->addSpacing(12);
+    bottomLayout->addSpacing(8);
     bottomLayout->addWidget(currentPageLabel);
-    bottomLayout->addSpacing(12);
+    bottomLayout->addSpacing(8);
     bottomLayout->addWidget(previousPageButton);
     bottomLayout->addWidget(nextPageButton);
     mainLayout->addLayout(bottomLayout);
@@ -255,6 +280,7 @@ void TimeLine::updateCurrentPageNumber()
 
 
 
+
 /*********************************************************/
 /*********************************************************/
 /************************ SLOTS **************************/
@@ -303,6 +329,7 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
 
             QString itemVerb = singlePostMap["verb"].toString();
 
+            // This will hold the date when the post was deleted, or empty if it wasn't
             QString postIsDeleted = objectMap["deleted"].toString();
 
 
@@ -329,7 +356,9 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             QString postIsLiked;
             QString postIsShared;
             QString postSharedBy;
+            QString postSharedById;
 
+            bool postIsNew = false;
 
             postID = objectMap["id"].toString();
             if (newestPostID.isEmpty()) // only first time, for newest post
@@ -345,6 +374,7 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
                 }
             }
 
+
             if (!allNewPostsCounted)
             {
                 if (postID == this->previousNewestPostID)
@@ -353,7 +383,14 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
                 }
                 else
                 {
-                    ++newPostCount;
+                    // If post is NOT deleted, add it to the count
+                    if (postIsDeleted.isEmpty())
+                    {
+                        ++newPostCount;
+
+                        // Mark current post as new
+                        postIsNew = true;
+                    }
                 }
             }
 
@@ -371,7 +408,12 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
                 // Get original author data
                 authorMap = objectMap["author"].toMap();
 
-                postSharedBy = singlePostMap["actor"].toMap().value("displayName").toString();
+                postSharedBy = singlePostMap["actor"].toMap()["displayName"].toString();
+                postSharedById = singlePostMap["actor"].toMap()["id"].toString();
+                if (postSharedById.startsWith("acct:"))
+                {
+                    postSharedById.remove(0,5);
+                }
             }
             else
             {
@@ -391,7 +433,7 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             postAuthorAvatar = authorMap["image"].toMap().value("url").toString();
             postAuthorId = authorMap["id"].toString();
             // For Author ID, remove the first 5 characters from the field, if they are "acct:"
-            // (users have acct: in from of the ID, but services like OFG don't)
+            // (users have acct: in front of the ID, but services like OFG don't)
             if (postAuthorId.startsWith("acct:"))
             {
                 postAuthorId.remove(0,5);
@@ -531,7 +573,9 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             postData.insert("postIsLiked",        postIsLiked);
             postData.insert("postIsShared",       postIsShared);
             postData.insert("sharedBy",           postSharedBy);
+            postData.insert("sharedById",         postSharedById);
 
+            postData.insert("postIsNew",          postIsNew ? "true":"false");
 
 
             if (postIsDeleted.isEmpty())  // if post was NOT deleted
@@ -548,6 +592,9 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
 
                 this->splitter->setMinimumHeight(this->splitter->height()
                                                  + newPost->sizeHint().height());
+
+                connect(newPost, SIGNAL(postRead()),
+                        this, SLOT(decreaseUnreadPostsCount()));
             }
             else
             {
@@ -576,11 +623,12 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
     emit scrollToTop(); // ask mainWindow to scroll the QScrollArea containing the timeline to the top
 
     this->previousNewestPostID = newestPostID;
+    this->unreadPostsCount = newPostCount;
     qDebug() << "New posts:" << newPostCount << "; Newest post ID:" << previousNewestPostID;
 
     this->updateCurrentPageNumber();
 
-    emit timelineRendered(this->timelineType, newPostCount);
+    emit timelineRendered(this->timelineType, unreadPostsCount);
 
     qDebug() << "setTimeLineContents() /END";
 }
@@ -641,7 +689,11 @@ void TimeLine::setCommentsInPost(QVariantList commentsList, QString originatingP
             qDebug() << "Found originating Post; setting comments on it...";
             post->setComments(commentsList);
 
-            break;
+            // break;
+            /* Don't break, so comments get set in copies of the post too,
+               like if JohnDoe posted something and JaneDoe shared it soon
+               after, so both the original post and its shared copy are visible
+               in the timeline. */
         }
     }
 }
@@ -684,3 +736,17 @@ void TimeLine::goToNextPage()
 
     this->requestTimelinePage();
 }
+
+
+
+/*
+ * Decrease internal counter of unread posts (by 1), and inform
+ * the parent window, so it can update its tab titles
+ *
+ */
+void TimeLine::decreaseUnreadPostsCount()
+{
+    --unreadPostsCount;
+
+    emit unreadPostsCountChanged(this->timelineType, this->unreadPostsCount);
+}
