@@ -51,6 +51,7 @@ public:
 
     void updateCurrentPageNumber();
 
+
     enum TimelineTypes
     {
         TimelineTypeMain,
@@ -68,6 +69,7 @@ signals:
     void scrollToTop();
 
     void timelineRendered(int timelineType, int newPostCount);
+    void unreadPostsCountChanged(int timelineType, int newPostCount);
 
 public slots:
     void setTimeLineContents(QVariantList postList, int postsPerPage,
@@ -80,6 +82,8 @@ public slots:
     void goToPreviousPage();
     void goToNextPage();
 
+    void decreaseUnreadPostsCount();
+
 private:
     QVBoxLayout *mainLayout;
     QSplitter *splitter;
@@ -99,6 +103,7 @@ private:
 
     int timelineOffset;
     int postsPerPage;
+    int unreadPostsCount;
 
     QString previousNewestPostID;
 
