@@ -28,8 +28,7 @@ Timestamp::Timestamp(QObject *parent) : QObject(parent)
 
 
 /*
- *  returns " 07-02-2012 "
- *          " 01:32:02   ", adjusted to local time
+ *  returns " 07-02-2012 - 01:32:02", adjusted to local time
  *
  *  isoTime = ISO 8601, UTC, like "2012-02-07T01:32:02Z"
  */
@@ -41,7 +40,7 @@ QString Timestamp::localTimeDate(QString isoTime)
 
     QString localTimeDateString;
     localTimeDateString = dateTimeConverter.date().toString();
-    localTimeDateString.append("\n");
+    localTimeDateString.append(", ");
     localTimeDateString.append(dateTimeConverter.time().toString());
 
 
